import React from 'react';
import Button from '@material-ui/core/Button';
import TextField from "@material-ui/core/TextField";
import List from '@material-ui/core/List';
//import ListItem from '@material-ui/core/ListItem';
import ListItemText from '@material-ui/core/ListItemText';
import ListItemAvatar from '@material-ui/core/ListItemAvatar';
import MuiListItem from "@material-ui/core/ListItem";
import TablePagination from '@material-ui/core/TablePagination';
import Avatar from '@material-ui/core/Avatar';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogContentText from '@material-ui/core/DialogContentText';
import DialogTitle from '@material-ui/core/DialogTitle';
import { makeStyles, withStyles } from '@material-ui/core/styles';
import FolderIcon from '@material-ui/icons/Folder';
import {APIContext} from "./callWrapper";

const useStyles = makeStyles((theme) => ({
  h1: {
    '& .MuiTypography-root': {
        color: "#0F3F7B"},
  },
  dialogWindow: {
  }
}));

const ListItem = withStyles({
  root: {
    "&$selected": {
      backgroundColor: "#0F3F7B",
      color: "white",
      "& .MuiListItemIcon-root": {
        color: "white"
      }
    },
    "&$selected:hover": {
      backgroundColor: "#D6DCE1",
      color: "#192E4A",
      "& .MuiListItemIcon-root": {
        color: "#192E4A"
      }
    },
    "&$selected:active":  {
      backgroundColor: "#192E4A",
      color: "white",
      "& .MuiListItemIcon-root": {
        color: "white"
      }
    },
    "&$selected:focus":  {
      backgroundColor: "#192E4A",
      color: "white",
      "& .MuiListItemIcon-root": {
        color: "white"
      }
    },
    "&:hover": {
      backgroundColor: "#D6DCE1",
      color: "#192E4A",
      "& .MuiListItemIcon-root": {
        color: "#192E4A"
      }
    },
    "&:active": {
      backgroundColor: "#192E4A",
      color: "white",
      "& .MuiListItemIcon-root": {
        color: "white"
      }
    },
    "&:focus": {
      backgroundColor: "#192E4A",
      color: "white",
      "& .MuiListItemIcon-root": {
        color: "white"
      }
    }
  },
  selected: {}
})(MuiListItem);

/* It's a scrolldialog for adding user to a group. */
const AddAdminDialog = (props) => {
  const classes = useStyles();
  const [open, setOpen] = React.useState(false);
  const [scroll, setScroll] = React.useState('paper');
  const descriptionElementRef = React.useRef(null);
  const client = React.useContext(APIContext);
  const MEMBER_CONTEXT = "memberContext";
  const ADMIN_CONTEXT = "adminContext";

  const [UIState, setUIState] = React.useState({
    "user-add-search": "",
    "admin-switch": false,
    "userSearchPagination": {"page": 0, "rowsPerPage": 10},
    [MEMBER_CONTEXT]: {"page": 0, "rowsPerPage": 10},
    [ADMIN_CONTEXT]: {"page": 0, "rowsPerPage": 10}
  })
  const [userSearchQueueIndex, setUserSearchQueueIndex] = React.useState(null);
  const [userSearchList, setUserSearchList] = React.useState([]);
  const [addedUsers, setAddedUsers] = React.useState([]);
  const [selectedIndex, setSelectedIndex] = React.useState(1);

  /**
   * Opens the dialog for adding admins.
   */
  const handleClickOpen = () => { setOpen(true); };

  /**
   * Resets all the chosen members.
   */
  const handleAbbrechen = () => {
    setAddedUsers([]);
    setOpen(false);
  };

  /**
   * By clicking the button, all the selected users will be listed in the createGroup adminlist.
   */
  const handleBestaetigen = () => {
    props.parentAdminCallback(addedUsers);
    setAddedUsers([]);
    setOpen(false);
  }

  // /**
  //  * Updating group data: name, owner, members, admins.
  //  * @param {} force
  //  */
  // const updateGroup = (force) => {
  //   if (force || state["group"] == null) {
  //     client.getGroup(props.groupID).then(async (updated_group) => {
  //       let newState = {}
  //       newState["group"] = updated_group
  //       newState["owner"] = await client.getUser(updated_group.owner)
  //       newState["members"] = await client.getUsers(updated_group.group_members)
  //       newState["admins"] = await client.getUsers(updated_group.group_admins)
  //       setState(newState)
  //     })
  //     .catch((error) => {
  //       console.error(error)
  //     });
  //   }
  // }

  React.useEffect(() => {
    if (open) {
      const { current: descriptionElement } = descriptionElementRef;
      if (descriptionElement !== null) {
        descriptionElement.focus();
      }
    }
  }, [open]);

  React.useEffect(() => {
    console.log("userSearchList updated.");
  }, [userSearchList]);

  /**
   * Get user to list them up in "Nutzer hinzufügen". 
   */
  const getUser = () => { client.getUsers(); };

  /**
   * Method with functionalities of a searchfield for searchign users.
   * @param {string} username
   */
  const setUserSearchField = (username) => {
    let uncommittedState = JSON.parse(JSON.stringify(UIState));
    uncommittedState["user-add-search"] = username;
    setUIState(uncommittedState)
    if (userSearchQueueIndex != null) {
      clearTimeout(userSearchQueueIndex)
    }
    setUserSearchQueueIndex(setTimeout(() => {
      setUserSearchQueueIndex(null);
      client.getUserByName(username).then((matchingUsers) => {
        setUserSearchList(matchingUsers)
      })
      .catch((error) => console.log(error.response))
    }, 500))
  }

  /**
   * Method to build a pagination when there are to many users listed.
   * @param {number} page
   * @param {number} rowsPerPage
   * @param {number} paginationID
   */
  const setPagination = (page, rowsPerPage, paginationID) => {
      let uncommittedState = JSON.parse(JSON.stringify(UIState));
      uncommittedState[paginationID]["page"] = page
      uncommittedState[paginationID]["rowsPerPage"] = rowsPerPage
      setUIState(uncommittedState)
  }


  /**
   * Deletes a user from adminmemberlist.
   * @param {Object} member
   */
  const deleteUser = (member) => { setAddedUsers(addedUsers.filter(user => user.username !== member.username)); }

  /**
   * Adds clicked user in the list for building adminlist in CreateGroup. 
   * @param {Object} user
   */
  const clickUser = (user) => {
    if (addedUsers.indexOf(user) == -1) { setAddedUsers(addedUsers.concat(user)) }   
  }

  /**
   * Handle the click on a user in the list.
   * @param {event} event
   * @param {number} index
   */
  const handleListItemClick = (event, index) => { setSelectedIndex(index); };

  return (
    <div>
      <Button 
        onClick={handleClickOpen} 
        variant="contained" 
        component="label" 
        color="primary">Admin hinzufügen</Button>
      <Dialog
        open={open}
        onClose={handleAbbrechen}
        scroll={scroll}
        aria-labelledby="scroll-dialog-title"
        aria-describedby="scroll-dialog-description"
        className={classes.dialogWindow}
      >
        <DialogTitle id="scroll-dialog-title" className={classes.h1}>Fügen Sie einen/mehrere Admin zu Ihrer Gruppe hinzu</DialogTitle>
        <DialogContent dividers={scroll === 'paper'}>
          <DialogContentText
            id="scroll-dialog-description"
            ref={descriptionElementRef}
            tabIndex={-1}
          >
            <TextField 
              id='user-add-search' 
              label='Benutzername' 
              type='text'
              value={UIState["user-add-search"]} 
              onChange={(event) => {setUserSearchField(event.target.value)}}
            />

            <List>
                {userSearchList.slice(UIState["userSearchPagination"]["page"] * UIState["userSearchPagination"]["rowsPerPage"], (UIState["userSearchPagination"]["page"] + 1) * UIState["userSearchPagination"]["rowsPerPage"]).map((member) => (
                  <ListItem 
                    button 
                    className="userList" 
                    selected={selectedIndex === userSearchList.indexOf(member)} 
                    onClick={(event) => {handleListItemClick(event, userSearchList.indexOf(member)); clickUser(member)}}
                  >
                    <ListItemAvatar>
                      <Avatar>
                        <FolderIcon/> 
                      </Avatar>
                    </ListItemAvatar>
                    <ListItemText 
                      primaryTypographyProps={{color: "primary"}} 
                      primary={member.username} 
                      key={member.username}/
                    >
                  </ListItem>
                ))}
              </List>,
              <TablePagination 
                count={userSearchList.length}
                rowsPerPage={UIState["userSearchPagination"]["rowsPerPage"]}
                page={UIState["userSearchPagination"]["page"]}
                onChangePage={(event) => {
                 setPagination(event.target.value, UIState["userSearchPagination"]["rowsPerPage"], "userSearchPagination")
                }}
                onChangeRowsPerPage={(event) => {
                 setPagination(UIState["userSearchPagination"]["page"], event.target.value, "userSearchPagination")
                }}
              />

          </DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={handleAbbrechen} color="primary">
            Abbrechen
          </Button>
          <Button onClick={handleBestaetigen} color="primary">
            Bestätigen
          </Button>
        </DialogActions>
      </Dialog>
    </div>
  );
};

export default AddAdminDialog; 
