/**
 This react component is next to the root and the entrypoint for UI components.
 It contains the drawer, the header (AppBar), the banner and the footer.
 All other components will be imported: content of the page.
 */

import React, {useEffect, useState} from 'react';
import {
  BrowserRouter as Router,
  Routes,
  Route,
  Link,
} from "react-router-dom";
import clsx from 'clsx';

/* Material UI Core */
import {makeStyles, useTheme} from '@material-ui/core/styles';
import Drawer from '@material-ui/core/Drawer';
import AppBar from '@material-ui/core/AppBar';
import Toolbar from '@material-ui/core/Toolbar';
import List from '@material-ui/core/List';
import CssBaseline from '@material-ui/core/CssBaseline';
import Typography from '@material-ui/core/Typography';
import Divider from '@material-ui/core/Divider';
import IconButton from '@material-ui/core/IconButton';
import ListItem from '@material-ui/core/ListItem';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import Button from '@material-ui/core/Button';
import ListItemSecAct from '@material-ui/core/ListItemSecondaryAction';
import Modal from '@material-ui/core/Modal';

/* Material UI Icons */
import MenuIcon from '@material-ui/icons/Menu';
import ChevronLeftIcon from '@material-ui/icons/ChevronLeft';
import ChevronRightIcon from '@material-ui/icons/ChevronRight';
import InboxIcon from '@material-ui/icons/MoveToInbox';
import AccountCircle from "@material-ui/icons/AccountCircle";
import Dashboard from "@material-ui/icons/Dashboard";
import Group from "@material-ui/icons/Group";
import InfoIcon from '@material-ui/icons/Info';
import HelpIcon from '@material-ui/icons/Help';
import NotificationsIcon from '@material-ui/icons/Notifications';

/* External installed Components*/
import SearchBar from "material-ui-search-bar";

/* Images, SVGs, etc. */
import Banner from './images/banner.png';

/* Selfcreated Components */
import MainContent from './MainContent';
import CollapseList from './CollapseList';
import CollapseListDashboard from './CollapseListDashboard';
import CollapseListGroup from './CollapseListGroup';
import AddChartMenu from './AddChartMenu';
import AddDashboardMenu from './AddDashboardMenu';
import AddGroupsMenu from './AddGroupsMenu';
import UserDashboard from './UserDashboard';
import CreateDashboard from './CreateDashboard';
import CreateGroup from './CreateGroup';
import {Footer, footerRoutes} from './components/Footer'

import {createClient, APIContext} from "./callWrapper";
import Login from "./Login";
import {ChartRoute} from "./ChartRender";
import ChartCreation from  "./ChartCreation";

import {BACKEND_URL} from "./config.json"
import PlaceholderComponent from "./PlaceholderComponent";
import GroupComponent from "./GroupComponent";
import {GroupRoute} from "./GroupDetails";
import ProfilePageView from "./ProfilePageView";
import Register from "./Register";

import {DashboardRoute} from "./DashboardEditor";

const drawerWidth = 240;

const useStyles = makeStyles((theme) => ({
  root: {
    display: 'flex',
    'min-height': '100vh',
    position: 'relative',
    'padding-bottom': '60px',
    'flex-direction': 'column',
  },
  appBar: {
    zIndex: theme.zIndex.drawer + 1,
    transition: theme.transitions.create(['width', 'margin'], {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.leavingScreen,
    }),
  },
  appBarShift: {
    marginLeft: drawerWidth,
    width: `calc(100% - ${drawerWidth}px)`,
    transition: theme.transitions.create(['width', 'margin'], {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.enteringScreen,
    }),
  },
  nested: {
        paddingLeft: theme.spacing(4),
      },
  menuButton: {
    marginRight: 36,
  },
  hide: {
    display: 'none',
  },
  drawer: {
    width: drawerWidth,
    flexShrink: 0,
    whiteSpace: 'nowrap',
  },
  drawerOpen: {
    width: drawerWidth,
    transition: theme.transitions.create('width', {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.enteringScreen,
    }),
  },
  drawerClose: {
    transition: theme.transitions.create('width', {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.leavingScreen,
    }),
    overflowX: 'hidden',
    width: theme.spacing(7) + 1,
    [theme.breakpoints.up('sm')]: {
      width: theme.spacing(9) + 1,
    },
  },
  toolbar: {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'flex-end',
    padding: theme.spacing(0, 1),
    // necessary for content to be below app bar
    ...theme.mixins.toolbar,
  },
  content: {
    flexGrow: 1,
    padding: theme.spacing(3),
  },
  paper: {
    position: 'absolute',
    width: 400,
    backgroundColor: theme.palette.background.paper,
    border: '2px solid #000',
    boxShadow: theme.shadows[5],
    padding: theme.spacing(2, 4, 3),
  },
  pushActive: {
    transition: 'margin-left 450ms cubic-bezier(0.23, 1, 0.32, 1)',
    marginLeft: 240,
  },
  pushInactive: {
    transition: 'margin-left 450ms cubic-bezier(0.23, 1, 0.32, 1)',
    marginLeft: 72,
  }
}));


function doSomethingWith(value) {
  return null;
}

export default function App() {

  return (
    <APIContext.Provider value={createClient(BACKEND_URL)}>
      <Router>
        <MiniDrawer/>
      </Router>
    </APIContext.Provider>
  )
}


function MiniDrawer() {
  const classes = useStyles();
  const theme = useTheme();
  const [open, setOpen] = React.useState(false);
  const [loginModalOpen, setloginModalOpen] = React.useState(false);
  const [loginState, setLoginState] = React.useState(false);
  const [openCharts, setOpenCharts] = React.useState(false);
  const [chart, setChart] =  React.useState("");
  const [openDashboard, setOpenDashboard] = React.useState(false);
  const [dashboard, setDashboard] = React.useState("");
  const [openGroups, setOpenGroups] = React.useState(false);
  const windowSize = useWindowSize();

  let showAddBtns = false;
  let chartName = "";
  let contentPushStyle = "pushInactive";
  
  const handleClickCharts = () => {
    setOpenCharts(!openCharts);
  };

    const handleClickGroups = () => {
    setOpenGroups(!openGroups);
  };

  const callbackFunc = (childData) => {
    chartName = childData;
    setChart(childData);
  };

  const handleDrawerOpen = () => {
    showAddBtns = true;
    setOpen(true);
    console.log("Drawer opened + showAddBtns: " + showAddBtns);
  };

  const handleDrawerClose = () => {
    showAddBtns = false;
    setOpen(false);
    console.log("Drawer closed + showAddBtns: " + showAddBtns);
  };

  const updateLoginState = (state) => {
    if(loginState !== state) {
      setLoginState(state);
    }
  };


  React.useEffect(() => {
    setOpenCharts(openCharts);
    setOpenGroups(openGroups);
    setLoginState(loginState);
  }, [loginState, openCharts, openGroups, open]);


  const publicChartID = 5
  const tree = {
    horizontal: false,
    DEBUG: 0,
    aspect: [900, 600], //[width, size]
    c1: {
      split: {
        horizontal: true,
        DEBUG: 0,
        aspect: [200, 100], //[width, size]
        c1: {
          split: {
            horizontal: false,
            DEBUG: 1,
            aspect: [100, 50],
            c1: {
              generatorName: "chart",
              args: {chartID: publicChartID}
            },
            c2: {
              generatorName: "chart",
              args: {chartID: publicChartID}
            },
          }
        },
        c2: {
          split: {
            horizontal: false,
            DEBUG: 2,
            aspect: [100, 50],
            c1: {
              generatorName: "chart",
              args: {chartID: publicChartID}
            },
            c2: {
              generatorName: "chart",
              args: {chartID: publicChartID}
            },
          }
        }
      }
    }
  }


  // Hook
  function useWindowSize() {

    // Initialize state with undefined width/height so server and client renders match
    const [windowSize, setWindowSize] = useState({
      width: undefined,
      height: undefined,
    });

    useEffect(() => {
      // Handler to call on window resize
      function handleResize() {
        // Set window width/height to state
        setWindowSize({
          width: window.innerWidth,
          height: window.innerHeight,
        });
      }

      // Add event listener
      window.addEventListener("resize", handleResize);

      // Call handler right away so state gets updated with initial window size
      handleResize();

      // Remove event listener on cleanup
      return () => window.removeEventListener("resize", handleResize);
    }, []); // Empty array ensures that effect is only run on mount

    return windowSize;
  }

  return (
    <div className={classes.root} id="drawer">
      <CssBaseline/>
      <AppBar
        position="fixed"
        className={clsx(classes.appBar, {
          [classes.appBarShift]: open,
        })}
      >
        <Toolbar>
          <IconButton
            color="inherit"
            aria-label="open drawer"
            onClick={handleDrawerOpen}
            edge="start"
            className={clsx(classes.menuButton, {
              [classes.hide]: open,
            })}
          >
            <MenuIcon/>
          </IconButton>
          <Typography variant="h6" noWrap>
            IVOD - Interactive Visualization of Open Data
          </Typography>
          {/*<SearchBar
            className="SearchBar"
            value={null}
            onChange={(newValue) => this.setState({value: newValue})}
            onRequestSearch={() => doSomethingWith(this.state.value)}
          />
          <NotificationsIcon
            className="HeaderNotification"
          />*/}
          <APIContext.Consumer>
            {api => {
              let displayName = "login/signup";
              api.addLoginStatusListener("MiniDrawer", updateLoginState)
              if(api.user) {
                displayName = api.user.username
              }
              return (
                <Button
                  id="HeaderLoginBtn"
                  variant="contained"
                  color="default"
                  className={classes.button}
                  endIcon={<AccountCircle>{displayName}</AccountCircle>}
                  onClick={() => {
                    if(loginState) {
                      api.logout()
                    } else {
                      setloginModalOpen(true);
                    }
                  }}
                >
                  {displayName}
                </Button>)
            }}
          </APIContext.Consumer>
          <Modal
            open={loginModalOpen}
            onClose={() => {
              setloginModalOpen(false)
            }}
            aria-labelledby="simple-modal-title"
            aria-describedby="simple-modal-description"
          >
            <div style={{top: '30%', left: '30%',}} className={classes.paper}>
              <Login onSuccess={() => {
                setloginModalOpen(false);
              }}/>
              <br/>
              <Register />
            </div>
          </Modal>

        </Toolbar>
      </AppBar>
      <Drawer
        variant="permanent"
        className={clsx(classes.drawer, {
          [classes.drawerOpen]: open,
          [classes.drawerClose]: !open,
        })}
        classes={{
          paper: clsx({
            [classes.drawerOpen]: open,
            [classes.drawerClose]: !open,
          }),
        }}
      >
        <div className={classes.toolbar}>
          <IconButton onClick={handleDrawerClose}>
            {theme.direction === 'rtl' ? <ChevronRightIcon/> : <ChevronLeftIcon/>}
          </IconButton>
        </div>
        <Divider/>
        <List>
          <ListItem button key="Account" component={Link} to={{
            pathname: "/account",
            state: {}
          }}>
            <ListItemIcon><AccountCircle/></ListItemIcon>
            <ListItemText primary="Account"/>
          </ListItem>
          <ListItem button key="Charts" onClick={handleClickCharts} component={Link} to={{
            pathname: "/",
            state: {}
          }}>

            <ListItemIcon><InboxIcon/></ListItemIcon>
            <ListItemText primary="Charts"/>
            <ListItemSecAct>
              <IconButton edge="end" aria-label="add">
                {open && loginState ? <AddChartMenu plotnameCallback={callbackFunc} /> : null}
              </IconButton>
            </ListItemSecAct>
          </ListItem>
          <ListItem button key="Dashboards" component={Link} to={{
            pathname: "/dashboards",
            state: {}
          }}>
            <ListItemIcon><Dashboard/></ListItemIcon>
            <ListItemText primary="Dashboards"/>
            <ListItemSecAct>
              <IconButton edge="end" aria-label="add">
                {open && loginState ? <AddDashboardMenu /> : null}
              </IconButton>
            </ListItemSecAct>
          </ListItem>
          {dashboard.length > 0 ? <CollapseListDashboard name={dashboard} open={openDashboard}/> : null}
          
          <ListItem button key="Groups" onClick={handleClickGroups} component={Link} to={{
            pathname: "/groups",
            state: {}
          }}>
            <ListItemIcon><Group/></ListItemIcon>
            <ListItemText primary="Groups"/>
            <ListItemSecAct>
              <IconButton edge="end" aria-label="add" >
                {open && loginState ? <AddGroupsMenu /> : null}
              </IconButton>
            </ListItemSecAct>
          </ListItem>
        </List>
        <Divider/>
        <List>
          <ListItem button key="Info" component={Link} to={{
            pathname: "/info",
            state: {}
          }}>
            <ListItemIcon><InfoIcon/></ListItemIcon>
            <ListItemText primary="Info"/>
          </ListItem>
          <ListItem button key="Help" component={Link} to={{
            pathname: "/help",
            state: {}
          }}>
            <ListItemIcon><HelpIcon/></ListItemIcon>
            <ListItemText primary="Help"/>
          </ListItem>
        </List>
      </Drawer>
      <div className={open ? classes.pushActive : classes.pushInactive}>
        <img class="banner" src={Banner} alt="Page-Banner"/>
        <main className={classes.content}>
          <Routes>
            <Route exact path="/" element={<MainContent/>} />
            <Route path="/upload" element={<ChartCreation />} />
            <Route path="/chart/:chartID" element={<ChartRoute />} />
            {/*TODO: Replace Placeholders with actual content*/}
            <Route path="/account" element={<ProfilePageView />} />

            <Route path="/dashboard/:dashboardID" element={<DashboardRoute dimensions={windowSize}/>}/>
            <Route path="/create-dashboard" element={<CreateDashboard />}/>
            <Route path="/dashboards" element={<UserDashboard />}/>
            
            <Route path="/groups" element={<GroupComponent />}/>
            <Route path="/create-group" element={<CreateGroup />} />
            <Route path="/group/:groupID" element={<GroupRoute/> }/>

            {/*TODO: Set actual content*/}
            <Route path="/info" element={<PlaceholderComponent/> }/>
            <Route path="/help" element={<PlaceholderComponent/> }/>

            { footerRoutes().map( (route) => {
              return route
            })}
            <Route path="*" element={<h1> Page not found! </h1>} />
          </Routes>
        </main>
        <Footer/>
      </div>
    </div>
  );
}
