import React, {useState, useRef} from "react";
import MenuItem from '@material-ui/core/MenuItem';
import {APIContext} from './callWrapper'
import Select from "@material-ui/core/Select";
import {Button, CircularProgress, Divider, TextField} from "@material-ui/core";
import Checkbox from '@material-ui/core/Checkbox';
import {FormControlLabel} from '@material-ui/core';
import {useNavigate} from "react-router-dom";
import {makeStyles} from '@material-ui/core/styles';
import Grid from '@material-ui/core/Grid';
import {Alert} from "@mui/material";
import Paper from '@material-ui/core/Paper';
import Typography from "@material-ui/core/Typography";



const ChartCreation = () => {

  const STATUS_NONE = 0
  const STATUS_CREATING = 1

  const useStyles = makeStyles((theme) => ({
    h2: {
      color: "black",
      "font-weight": "500",
    },
    paper: {
      padding: 25,
      margin: 12,
    },
    typography: {
      color: 'black',
      position: 'relative',
      bottom: -4,
    },
    label: {
      color: 'black !important',
    },
    textField: {
    position: 'relative',
    bottom: 14,
  }
  }));
  const classes = useStyles();

  const client = React.useContext(APIContext)

  const navigate = useNavigate()

  const [creationStatus, setCreationStatus] = React.useState(STATUS_NONE)
  const [committedState, setstate] = React.useState({
    dropdown: {
      'chart-creation-datasource-select': {
        open: false,
        value: "",
      },
      'chart-creation-charttype-select': {
        open: false,
        value: "",
      }
      ,
      'chart-creation-visibility-select': {
        open: false,
        value: 0,
      }
    },
    last_opened: "",
    datasources: undefined,
    charttypes: {},
    new_datasource_name: "",
    new_chart_name: "",
    downloadCheckbox: false,
    previous: {},
  });
  const uncommittedState = JSON.parse(JSON.stringify(committedState));
  const commitState = () => {
    setstate(uncommittedState);
  };

  const getIndexForID = (id) => {
    /* Find datasources in dropdown menu by its database id */

    //If there are no datasources getting an index of them is not possible
    if (uncommittedState.datasources === undefined) {
      return undefined;
    }
    // Either returns a single-element list containing the index or an empty list, in which case the lookup will return undefined
    return uncommittedState.datasources.filter((ds, index) => {
      if (ds.id == id) {
        return index;
      }
    })[0];
  }

  const getIDforIndex = (index) => {
    /* Get datasource database index from dropdown menu index */
    return uncommittedState.datasources[index].id
  }

  const handleChange = (event) => {
    let list = uncommittedState.dropdown[uncommittedState.last_opened];
    list.value = event.target.value;
    commitState()
  };

  const handleClose = (event) => {
    let list = uncommittedState.dropdown[uncommittedState.last_opened]
    list.open = false;
    commitState()
  };

  const handleOpen = (event) => {
    uncommittedState.last_opened = event.target.id;
    uncommittedState.dropdown[uncommittedState.last_opened]['open'] = true;
    commitState()
  };

  const handleFileUpload = e => {
    /* Datasource file selected, try to upload and select */
    const visibility = 0; //FIXME: Get visibility from UI, safe default 0=private
    client.createDatasourceFromFile(e.target.files[0], uncommittedState.new_datasource_name, visibility, (response) => {
        reloadDatatasources(response["data"]["id"])
      },
      console.error);
  };

  const reloadDatatasources = (selected) => {
    /* Reload all datasources from the server,
    if "selected" is a datasource id an attempt is made to select the entry in the dropdown menu */

    // Perform lookup
    client.getDatasources().then((datasources) => {
      uncommittedState.datasources = datasources;
      uncommittedState.charttypes = {}
      // Attempt to display correct datasource
      if (selected != undefined) {
        uncommittedState.dropdown['chart-creation-datasource-select']['value'] = getIndexForID(selected)
      }
      console.log(datasources)
      commitState();
    })
  };

  const showCreationStatus = () => {
    if(creationStatus === STATUS_NONE) {
      return undefined
    } else if(creationStatus === STATUS_CREATING) {
      return <CircularProgress />
    } else {
      //TODO: Add error message
      return <Alert severity="error"> Error {creationStatus} </Alert>
    }
  }

  const createChart = () => {
    /* Try to create a new chart */

    const datasourceIndex = uncommittedState.dropdown['chart-creation-datasource-select']['value'];
    const charttypeIndex = uncommittedState.dropdown['chart-creation-charttype-select']['value']
    setCreationStatus(STATUS_CREATING)
    client.createChart(
      getIDforIndex(datasourceIndex),
      uncommittedState.charttypes[datasourceIndex][charttypeIndex],
      {},
      uncommittedState.new_chart_name,
      uncommittedState.downloadCheckbox,
      uncommittedState.dropdown['chart-creation-visibility-select']['value']
    ).then((chart) => {
      // If the creation was successful, redirect to rendering of newly created chart
      navigate(`/chart/${chart.id}`)
    }).catch((error) => {
      if(error.response) {
        setCreationStatus(error.response.status)
      }else {
        setCreationStatus("Something went wrong")
        console.error(error)
      }
    })
  }

  React.useEffect(() => {
    if (uncommittedState.datasources === undefined) {
      reloadDatatasources();
    } else {
      //datasources loaded
      const datasourceIndex = uncommittedState.dropdown['chart-creation-datasource-select']['value']
      //check if a datasource is selected
      if (uncommittedState.datasources[datasourceIndex]) {
        //check if charttypes for datasource are cached, otherwise get them
        if (!uncommittedState.charttypes[datasourceIndex]) {
          client.getChartTypes(getIDforIndex(datasourceIndex)).then((types) => {
            uncommittedState.charttypes[datasourceIndex] = types;
            console.log(types)
            commitState();
          }).catch((error) => {
            console.log(error)
          })
        }
      }
    }
  })


  return (
    <div className={classes.root}>
      <Paper className={classes.paper}>
      <Grid container spacing={3}>
        <Grid item xs={12}>
          <h2 className={classes.h2}>Erstellen Sie eine Chart mit eigenen Datensätzen</h2>
        </Grid>
        <Grid item xs={2}>
          <Typography className={classes.typography}>1. Datenquellennamen eingeben</ Typography>
        </Grid>
        <Grid item xs={10}>
          <TextField
            className={classes.textField}
            id='chart-creation-new-datasource-name'
            label='Name für Datenquelle'
            type='text'
            value={uncommittedState.new_datasource_name}
            onChange={(event) => {
              uncommittedState.new_datasource_name = event.target.value;
              commitState();
            }}
          />
        </Grid>
        <Grid item xs={2}>
          <Typography className={classes.typography}>2. Datenquelle auswählen</ Typography>
        </Grid>
        <Grid item xs={10}>
          <Button variant="contained" component="label" color="primary"
                  disabled={uncommittedState.new_datasource_name === ""}>
            Daten hinzufügen
            <input type="file" hidden onChange={handleFileUpload}/>
          </Button>
        </Grid>
        <Grid item xs={12}>
        </Grid>
        <Grid item xs={12}>
        </Grid>
        <Grid item xs={12}>
        </Grid>
        <Grid item xs={2}>
          <Typography className={classes.typography}>3. hochgeladene Datenquelle auswählen</ Typography>
        </Grid>
        <Grid item xs={3}>
          <Select
            labelId="chart-creation-datasource-select-label"
            id="chart-creation-datasource-select"
            className={classes.select}
            open={uncommittedState.dropdown['chart-creation-datasource-select']['open']}
            onClose={handleClose}
            onOpen={handleOpen}
            value={uncommittedState.dropdown['chart-creation-datasource-select']['value']}
            onChange={handleChange}
          >
            {
              uncommittedState.datasources ?
                uncommittedState.datasources.map((object, index) => {
                  return <MenuItem value={index}> {object.datasource_name} </MenuItem>;
                }) : undefined

            }
          </Select>
        </Grid>
        <Grid item xs={7}>
        </Grid>
        <Grid item xs={12}>
        </Grid>
        {uncommittedState.dropdown['chart-creation-datasource-select']['value'] !== "" ?
          <React.Fragment>
            <Grid item xs={2}>
              <Typography className={classes.typography}>Art des Charts</ Typography>
            </Grid>
            <Grid item xs={10}>
              <Select
                labelId="chart-creation-charttype-select-label"
                id="chart-creation-charttype-select"
                className={classes.select}
                open={uncommittedState.dropdown['chart-creation-charttype-select']['open']}
                onClose={handleClose}
                onOpen={handleOpen}
                value={uncommittedState.dropdown['chart-creation-charttype-select']['value']}
                onChange={handleChange}
              >
                {

                  uncommittedState.charttypes[uncommittedState.dropdown['chart-creation-datasource-select']['value']] ?
                    uncommittedState.charttypes[uncommittedState.dropdown['chart-creation-datasource-select']['value']].map((object, index) => {
                      return <MenuItem value={index}> {object} </MenuItem>;
                    }) : undefined

                }
              </Select>
            </Grid>
            <Grid item xs={2}>
              <Typography className={classes.typography}>Chartname</ Typography>
            </Grid>
            <Grid item xs={3}>
              <TextField
                className={classes.textField}
                id='chart-creation-new-chart-name'
                label='Name für Chart'
                type='text'
                value={uncommittedState.new_chart_name}
                onChange={(event) => {
                  uncommittedState.new_chart_name = event.target.value;
                  commitState();
                }}
              />
            </Grid>
            <Grid item xs={7}>
            </Grid>

            <Grid item xs={2}>
              <Typography className={classes.typography}>Downloadbar</ Typography>
            </Grid>
            <Grid item xs={8}>
              {/*<FormControlLabel className={classes.label}
                control={
                  <Checkbox
                    checked={uncommittedState.downloadCheckbox}
                    onChange={(event) => {
                      uncommittedState.downloadCheckbox = event.target.checked;
                      commitState()
                    }}
                    name="downloadCheckbox"
                    color="primary"
                  />
                }
                />*/}
              <Select
                labelId="chart-creation-visibility-select-label"
                id="chart-creation-visibility-select"
                className={classes.select}
                open={uncommittedState.dropdown['chart-creation-visibility-select']['open']}
                onClose={handleClose}
                onOpen={handleOpen}
                value={uncommittedState.dropdown['chart-creation-visibility-select']['value']}
                onChange={handleChange}
              >
                <MenuItem value={0}> Privat </MenuItem>
                <MenuItem value={1}> Geteilt </MenuItem>
                <MenuItem value={2}> Teil-Öffentlich </MenuItem>;
                <MenuItem value={3}> Öffentlich </MenuItem>;
              </Select>
            </Grid>
            <Grid item xs={2}>
              <Button variant="contained" component="label" color="primary" onClick={createChart}
                      disabled={uncommittedState.dropdown['chart-creation-charttype-select']['value'] === ""
                      || uncommittedState.new_chart_name === ""}>
                Neue Chart erzeugen
              </Button>
              { showCreationStatus() }
            </Grid>
          </React.Fragment>
          : ""}
      </Grid>
      </Paper>
    </div>
  );
};

export default ChartCreation;
