import Grid from '@material-ui/core/Grid';
import Card from '@material-ui/core/Card';
import CardHeader from '@material-ui/core/CardHeader';
import CardMedia from '@material-ui/core/CardMedia';
import Avatar from '@material-ui/core/Avatar';
import {makeStyles} from '@material-ui/core/styles';

import CardActions from '@material-ui/core/CardActions';
import {Link} from "react-router-dom";
import React from "react";
import RemoveIcon from '@material-ui/icons/Remove';
import ShareIcon from '@mui/icons-material/Share';
import {IconButton} from "@material-ui/core";
import {APIContext} from "./callWrapper";
import Modal from "@material-ui/core/Modal";
import ShareDialog from "./ShareDialog";

const useStyles = makeStyles((theme) => ({
  chartPreviewHeaderContent: {
    width: '80%',
  },
  chartPreviewMedia: {
    height: 0, paddingTop: '56.25%'
  },
  paper: {
    position: 'absolute',
    // width: 400,
    backgroundColor: theme.palette.background.paper,
    border: '2px solid #000',
    boxShadow: theme.shadows[5],
    padding: theme.spacing(2, 4, 3),
    overflowY: "auto",
    height: "50%"
  },
  iconButtonRoot: {
    'background-color': 'gray'
  }
}));


function ChartPreview(props) {

  const client = React.useContext(APIContext)

  const classes = useStyles();

  const [shareModalOpen, setShareModalOpen] = React.useState(false)

  return <Grid item xs={4}>
    <Modal
            open={shareModalOpen}
            onClose={() => {
              setShareModalOpen(false)
            }}
            aria-labelledby="simple-modal-title"
            aria-describedby="simple-modal-description"
          >
            <div style={{top: '30%', left: '30%',}} className={classes.paper}>
              <ShareDialog ressourceID={props.chartObject.id} ressourceType={"chart"} />
            </div>
          </Modal>
    <Card>
      <CardHeader
        classes={{
          content: classes.chartPreviewHeaderContent,
        }}
        avatar={
          <Avatar alt="USER_PFP_PLACEHOLDER" src={props.pfp}>
          </Avatar>
        }
        /*
        action={
          <IconButton aria-label="settings">
            <MoreVertIcon />
          </IconButton>
        }
        */
        title={props.chartObject.chart_name}
        titleTypographyProps={{'noWrap': true, 'color': 'textPrimary'}}
      />
      <CardMedia
        className={classes.chartPreviewMedia}
        image={props.preview}
        title={props.chartObject.chart_name}
        component={Link} to={{
        pathname: `/chart/${props.chartObject.id}`,
        state: {}
      }}
      />
      <CardActions disableSpacing>

        < IconButton aria-label="add to favorites"
                     color="primary"
                     disabled={!(client.user?.id === props.chartObject.owner)}
                     onClick={() => {
                       client.deleteChart(props.chartObject.id).then(() => {
                         props.onChartDelete(props.chartObject.id)
                       })
                     }
                     }
                     classes={{
                       root: classes.iconButtonRoot
                     }}>
          <RemoveIcon/>
        </IconButton>
        {/*<IconButton aria-label="add to favorites" color="primary" classes={{*/}
        {/*    root: classes.iconButtonRoot*/}
        {/*}}>*/}
        {/*  <FavoriteIcon />*/}
        {/*</IconButton>*/}
        <IconButton aria-label="share"
                    onClick={() => {
                      setShareModalOpen(true)
                    }}
                    disabled={!(client.user?.id === props.chartObject.owner)}
                    classes={{
            root: classes.iconButtonRoot
        }}>
          <ShareIcon />
        </IconButton>
      </CardActions>
    </Card>
  </Grid>;
}

export default ChartPreview

/*  return <div>
    <div> <img src={props.pfp} alt={"Profile Picture"} width={32} height={32}/> {props.chartDescription} </div> 
    <img src={props.preview} alt={props.chartDescription} width={300} height={300}/>
    <div> TODO: Funktionstasten</div>
  </div>;*/