import React from 'react';
import {APIContext} from './callWrapper'
import {useParams} from "react-router-dom";

export function ChartRoute() {
    const params = useParams()
    const [chartID, setChartID] = React.useState(params["chartID"])

    React.useEffect( () => {
        console.log("Updating component")
        setChartID(params["chartID"])
    }, [params])

    return <ChartRender key={chartID} chartID={chartID} />

}

export default function ChartRender({chartID, dimensions}) {

    const client = React.useContext(APIContext)

    const [divID, setdivID] = React.useState(Math.round(Math.random() * Number.MAX_SAFE_INTEGER));
    const [config, setConfig] = React.useState(null);
    const [chartObject, setChartObject] = React.useState(null);

    const getCurrentConfig = () => {
        return config
    }

    const loadScriptTags = (libraryURL) => {
        /* Determine which scripts are missing from runtime */

        const requiredScripts = libraryURL;
        const head = document.getElementsByTagName('head')[0];
        const loadedScripts = head.querySelectorAll('script');

        //Filter out already loaded scripts
        for (const script of loadedScripts) {
            // Search be script source
            let index = requiredScripts.indexOf(script.src);
            if(index !== -1) {
                // Script already loaded, remove from remove from requirements
                requiredScripts.splice(index, 1);
            }
        }
        // Save missing libraries to state, next React.useEffect will load them
        return requiredScripts
    }

    const rescale = () => {
        const shallowConfigCopy = Object.assign({}, config)
        if(dimensions) {
            shallowConfigCopy["width"] = dimensions[0]*0.9
            shallowConfigCopy["height"] = shallowConfigCopy["width"] / shallowConfigCopy['webfrontend_aspect_ratio']
        }
        shallowConfigCopy['webfrontend_initial_rescale'] = true // Workaround to redraw on first dimension change
        setConfig(shallowConfigCopy)
    }

    const addScriptTags = (loadOrder) => {
        //Add missing scripts

        // Check if there are libraries left to load
        // When used normally, this should not occur
        while(loadOrder.length > 0) {
            // Get head-object
            const head = document.getElementsByTagName('head')[0];
            // pop first script url
            const scriptURL = loadOrder.shift()
            // Insert script tag into head
            const scriptTag = document.createElement('script');
            scriptTag.src = scriptURL;
            head.appendChild(scriptTag)
            // Copy and save missing scripts (workaround for reference bug)
        }
    }

    const renderChart = () => {
        let chart = undefined;
        if(!chartObject) {
            //Dependencies loaded, create chart object
            chart = window.pive.createChartFromConfig(config);
            setChartObject(chart);
        } else {
            chart = chartObject
        }
        const rerenderChartOnChange = () => {
            if(chart.config !== getCurrentConfig()) {
                chart.setConfig(getCurrentConfig())
                chart.render(rerenderChartOnChange)
            }
        }
        chart.render(rerenderChartOnChange)
    }

    React.useEffect( () => {
        if(config !== null) {
            if(config['webfrontend_initial_rescale']) {
                renderChart()
            } else {
                rescale()
            }
        }
    }, [config])

    React.useEffect( () => {
        if(dimensions && config) {
            rescale()
        }
    }, [dimensions])

    React.useEffect( () => {
        console.log("ChartRender gets mounted")
        // Get required urls, add base.js as version manager for pive scripts
        client.getChartCodeURL(chartID).then((result)=>{
            addScriptTags(
                loadScriptTags([`${client.baseURL}/api/code/base.js`,result])
            );
            const configURL = client.getChartConfigURL(chartID);
            window.d3.json(configURL)
                .then((config) => {
                    // Override div-hook
                    config['div_hook'] = "id"+divID
                    config['webfrontend_aspect_ratio'] = (config['width']*1.0) / config['height']
                    config['webfrontend_initial_rescale'] = false
                    // Config is loaded, but pive dependency might not be available yet
                    // Poll every 100ms until dependency is available
                    const interval = setInterval(() => {
                        if (!(window.pive === undefined || window.pive[config.js_name.toLowerCase()] === undefined)) {
                            //FIXME: Will never be executed if wrong library is loaded, can this be exploited for DoS?
                            clearInterval(interval)
                            setConfig(config)
                        }
                    }, 100)
                });
        })
    }, [])

    //TODO:
    return (<div id={"id"+divID} style={{"background-color":"white"}}>
    </div>);
}