import React, {useEffect} from "react";
import { makeStyles } from '@material-ui/core/styles';
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemText from '@material-ui/core/ListItemText';
import Collapse from '@material-ui/core/Collapse';
import {APIContext} from "./callWrapper";
import {
  Link,
} from "react-router-dom";


/**
 * Component in the menu, collapses if user is a member of groups.
 * @param {props} props
 * @return {HTMLElement} collapselist with groups
 */
const CollapseListGroup = (props) => {
  const client = React.useContext(APIContext);
  const useStyles = makeStyles((theme) => ({
    nested: {
        paddingLeft: theme.spacing(4),
      },
  }));
  const classes = useStyles();
  const [userGroups, setUserGroups] = React.useState([]);

  /**
   *  Loads all available groups for the logged in user.
   */
  const loadGroups = () => {
    client.getGroups(true)
    .then((groups) => {  
      setUserGroups(groups.slice(0,3)); // gets the last 3 groups 
    })
    .catch((error) => {
      console.log(error.response);
    });
  };

  /**
   *  Run only once on mount and use loadgroups() when it's neccessary, otherwise endless loop. 
   */
  React.useEffect(() => {
    loadGroups();
  }, []);


  /* Returns the last 3 groups as a clickable, collapsable group-list. */
  return (<Collapse in={props.collapse} timeout="auto" unmountOnExit>
            <List component="div" disablePadding>
              {userGroups.map(group => (<ListItem button className={classes.nested} key={group.name} component={Link} to={{
                pathname: "/group/"+group.id,
                state: {}
              }}>
                  <ListItemText primary={group.name} />
                  </ListItem>))}
            </List>
           </Collapse>)
  }

export default CollapseListGroup;   
