import React from 'react';
import { makeStyles } from '@material-ui/core/styles';
import Grid from '@material-ui/core/Grid';
import MenuItem from '@material-ui/core/MenuItem';
import {APIContext} from './callWrapper'
import Select from "@material-ui/core/Select";
import {Button, TextField} from "@material-ui/core";
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemAvatar from '@material-ui/core/ListItemAvatar';
import ListItemText from '@material-ui/core/ListItemText';
import FolderIcon from '@material-ui/icons/Folder';
import Avatar from '@material-ui/core/Avatar';
import {useNavigate} from "react-router-dom";
import Paper from '@material-ui/core/Paper';
import Typography from "@material-ui/core/Typography";

const useStyles = makeStyles((theme) => ({
  root: {
    flexGrow: 1,
  },
  divider: {
    margin: theme.spacing(2, 0),
  },
  paper: {
    padding: 25,
    margin: 12,
  },
  typography: {
    color: 'black !important',
    position: 'relative',
    bottom: -4,
  },
  textField: {
    position: 'relative',
    bottom: 14,
  }
}));

const CreateDashboard = (props) => {
	const classes = useStyles();
  const client = React.useContext(APIContext)
  const navigate = useNavigate()
	const [committedState, setstate] = React.useState({
      dropdown: {
        'dashboard-creation-visibility-select': {
          open: false,
          value: 0,
        }
      },
      new_dashboard_name: "",
      dashboard_charts: [],
  });
  const uncommittedState = JSON.parse(JSON.stringify(committedState));

  /**
   * Changes uncommitted state to a comitted one.
   */
  const commitState = () => { setstate(uncommittedState); };
  
  /**
   * Handles the change of the select-element an its' value.
   * @param {event} event
   */
  const handleChange = (event) => {
    let list = uncommittedState.dropdown[uncommittedState.last_opened];
    list.value = event.target.value;
    commitState()
  };

  /**
   * Handles the close of the select-element.
   * @param {event} event
   */
  const handleClose = (event) => {
    let list = uncommittedState.dropdown[uncommittedState.last_opened]
    list.open = false;
    commitState()
  };


  /**
   * Handles the open of the select-element.
   * @param {event} event
   */
  const handleOpen = (event) => {
    uncommittedState.last_opened = event.target.id;
    uncommittedState.dropdown[uncommittedState.last_opened]['open'] = true;
    commitState()
  };

  /**
   * Creates a config for the dashboard with json-format depending on the count of charts.
   * These data are important for the dashboard structure including splits, heights and charts.
   * Generally it is a treebased structure.
   */
  const handleDashboardConfig = (chartsList) => {
    const chartsCount = chartsList.length;

    console.log("test in handleDashboardconfig");
    console.log(chartsList[0].id);

    switch(chartsCount) {
      case 1:
        return {
          horizontal: false,
          DEBUG: 0,
          aspect: [900, 600], //[width, size]
          c1: {
            generatorName: "chart",
            args: {chartID: chartsList[0].id}
          }
        };
      case 2:
        return  {
          horizontal: false,
          DEBUG: 0,
          aspect: [900, 600], //[width, size]
          c1: {
            split: {
              horizontal: true,
              DEBUG: 0,
              aspect: [200, 100], //[width, size]
              c1: {
                generatorName: "chart",
                args: {chartID: chartsList[0].id}
              },
              c2: {
                generatorName: "chart",
                args: {chartID: chartsList[1].id}
              }
            }
          }
        };
      case 3:
        return  {
          horizontal: false,
          DEBUG: 0,
          aspect: [900, 600], //[width, size]
          c1: {
            split: {
              horizontal: true,
              DEBUG: 0,
              aspect: [200, 100], //[width, size]
              c1: {
                split: {
                  horizontal: false,
                  DEBUG: 1,
                  aspect: [100, 50],
                  c1: {
                    generatorName: "chart",
                    args: {chartID: chartsList[0].id}
                  },
                  c2: {
                    generatorName: "chart",
                    args: {chartID: chartsList[1].id}
                  },
                }
              },
              c2: {
                generatorName: "chart",
                args: {chartID: chartsList[2].id}
              }
            }
          }
        };
      case 4:
        return {
          horizontal: false,
          DEBUG: 0,
          aspect: [900, 600], //[width, size]
          c1: {
            split: {
              horizontal: true,
              DEBUG: 0,
              aspect: [200, 100], //[width, size]
              c1: {
                split: {
                  horizontal: false,
                  DEBUG: 1,
                  aspect: [100, 50],
                  c1: {
                    generatorName: "chart",
                    args: {chartID: chartsList[0].id}
                  },
                  c2: {
                    generatorName: "chart",
                    args: {chartID: chartsList[1].id}
                  },
                }
              },
              c2: {
                split: {
                  horizontal: false,
                  DEBUG: 2,
                  aspect: [100, 50],
                  c1: {
                    generatorName: "chart",
                    args: {chartID: chartsList[2].id}
                  },
                  c2: {
                    generatorName: "chart",
                    args: {chartID: chartsList[3].id}
                  },
                }
              }
            }
          }
        };
    }
  }

  /**
   * Sends all dashboarddata to the server,if they're complete. 
   */
  const addDashboardClick = (event) => {

    const tree = {
    horizontal: false,
    aspect: [900, 600], //[width, size]
    c1: {}
  }  

    client.createDashboard(
        committedState.new_dashboard_name,
        tree,
        committedState.dropdown['dashboard-creation-visibility-select']['value']
    	).then((dashboard) => {
    		navigate(`/dashboard/${dashboard.id}`)
    	})
    	.catch((error) => {
    		console.log(error.response);
    	})
  };

	/**
	 * Deletes a chart from chartslist.
	 * @param {object} charts
	 */
	const deleteChart = (givenChart) => {
	  uncommittedState.dashboard_charts = uncommittedState.dashboard_charts.filter(chart => chart.chart_name !== givenChart.chart_name);
	  committedState.dashboard_charts = uncommittedState.dashboard_charts;
	  commitState();
	}

  /**
   * Builds dynamically a chartslist and will be shown in the view.
   * @return {HTMLElement} built memberlist
   */
  const buildChartList = (chartsList) => {
    if (chartsList != null) {
      return (<List>
          {chartsList.map((chart) => (
            <ListItem button onClick={() => {deleteChart(chart)} }>
              <ListItemAvatar>
                <Avatar>
                  <FolderIcon/> {/* TODO: Insert User Avatar here*/}
                </Avatar>
              </ListItemAvatar>
              <ListItemText primaryTypographyProps={{color: "primary"}} primary={chart.chart_name}/>
            </ListItem>
          ))}
        </List>) 
    }
  }

  /**
   * Adds a chart to the dashboard and avoids duplicate charts.
   * @param {object} chart
   * */
  const addCharts = (chart) => {
    let isAlreadyAdded = false;

    uncommittedState.dashboard_charts.map(chartInDashboard => {
      if(chart.id == chartInDashboard.id) {
        isAlreadyAdded = true;
      }
    })

    if (!isAlreadyAdded) {
      uncommittedState.dashboard_charts.push(chart);
      commitState();
      isAlreadyAdded = false;
    } else {
      console.log("chart is already added to dashboard.");
    }
  };

  React.useEffect(() => {

  }, []);

	return(
		<div className={classes.root}>
    <Paper className={classes.paper}>
	      <Grid container spacing={3}>
	        <Grid item xs={12}>
	          <h2>Erstellen Sie ein neues Dashboard</h2>
	        </Grid>

	        <Grid item xs={2}>
	          <Typography className={classes.typography}>Name des Dashboards</Typography>
	        </Grid>
	        <Grid item xs={10}>
	           <TextField 
	           	id='chart-creation-new-dashboard-name'
	           	className={classes.textField}
	           	label='Dashboardname' 
	           	type='text'
		        value={uncommittedState.new_dashboard_name} 
		        InputProps={{
			        className: classes.input,
			    }}
		        onChange={(event) => {
		          uncommittedState.new_dashboard_name = event.target.value;
		          commitState();
		        }}/>
	        </Grid>

	        <Grid item xs={2}>
	          <Typography className={classes.typography} >Dashboard-Sichtbarkeit</Typography>
	        </Grid>
	        <Grid item xs={8}>
	          	<Select
		          labelId="dashboard-creation-visibility-select-label"
		          id="dashboard-creation-visibility-select"
		          className={classes.select}
		          open={uncommittedState.dropdown['dashboard-creation-visibility-select']['open']}
		          onClose={handleClose}
		          onOpen={handleOpen}
		          value={uncommittedState.dropdown['dashboard-creation-visibility-select']['value']}
		          onChange={handleChange}
		        >
		          <MenuItem value={0}> Privat </MenuItem>
              <MenuItem value={1}> Geteilt </MenuItem>
              <MenuItem value={2}> Teil-Öffentlich </MenuItem>;
              <MenuItem value={3}> Öffentlich </MenuItem>;
        		</Select>
	        </Grid>

	        <Grid item xs={2}>
	          	<Button 
	          		onClick={addDashboardClick} 
	          		variant="contained" 
	          		component="label" 
	          		color="primary"
	          		disabled={committedState.new_dashboard_name.length === 0}
              >
              		Dashboard erstellen
            	</Button>
	        </Grid>
	      </Grid>
      </Paper>
    	</div>
	);
};

export default CreateDashboard;	