import React from 'react';
import {makeStyles} from '@material-ui/core/styles';
import Grid from '@material-ui/core/Grid';
import MenuItem from '@material-ui/core/MenuItem';
import {APIContext} from './callWrapper'
import Select from "@material-ui/core/Select";
import {Button, CircularProgress, TextField} from "@material-ui/core";
import {useNavigate} from "react-router-dom";
import AddUserDialog from "./AddUserDialog";
import AddAdminDialog from "./AddAdminDialog";
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemAvatar from '@material-ui/core/ListItemAvatar';
import ListItemText from '@material-ui/core/ListItemText';
import FolderIcon from '@material-ui/icons/Folder';
import Avatar from '@material-ui/core/Avatar';
import {Alert} from "@mui/material";
import Paper from '@material-ui/core/Paper';
import Typography from "@material-ui/core/Typography";

// todo: will boolean bei create group. für visible

const useStyles = makeStyles((theme) => ({
  root: {
    flexGrow: 1,
  },
  divider: {
    margin: theme.spacing(2, 0),
  },
  paper: {
    padding: 25,
    margin: 12,
  },
  typography: {
    color: 'black !important',
    position: 'relative',
    bottom: -4,
  },
  textField: {
    position: 'relative',
    bottom: 14,
  }
}));


/**
 * Component that allows to create groups with members, admins, name und visibility.
 * @param {props} props
 */
const CreateGroup = (props) => {

  const STATUS_NONE = 0
  const STATUS_CREATING = 1

  const [MEMBER_CONTEXT, ADMIN_CONTEXT, CHART_CONTEXT, DATASOURCE_CONTEXT] = ["memberContext", "adminContext", "chartContext", "datasourceContext"];
  const classes = useStyles();
  const client = React.useContext(APIContext)
  const navigate = useNavigate()
  const [creationStatus, setCreationStatus] = React.useState(STATUS_NONE)
  const [state, setState] = React.useState({group: null, owner: null, members: null, admins: null})
  const [deleteModalConfig, setDeleteModalConfig] = React.useState({open: false, obj: null, context: null})
  const [committedState, setstate] = React.useState({
    dropdown: {
      'group-creation-visibility-select': {
        open: false,
        value: 0,
      }
    },
    new_group_name: "",
    group_members: [],
    group_admins: [],
  });
  const uncommittedState = JSON.parse(JSON.stringify(committedState));

  /**
   * Changes uncommitted state to a comitted one.
   */
  const commitState = () => {
    setstate(uncommittedState);
  };

  /**
   * Handles the change of the select-element an its' value.
   * @param {event} event
   */
  const handleChange = (event) => {
    let list = uncommittedState.dropdown[uncommittedState.last_opened];
    list.value = event.target.value;
    commitState()
  };

  /**
   * Handles the close of the select-element.
   * @param {event} event
   */
  const handleClose = (event) => {
    let list = uncommittedState.dropdown[uncommittedState.last_opened]
    list.open = false;
    commitState()
  };

  /**
   * Handles the open of the select-element.
   * @param {event} event
   */
  const handleOpen = (event) => {
    uncommittedState.last_opened = event.target.id;
    uncommittedState.dropdown[uncommittedState.last_opened]['open'] = true;
    commitState()
  };

  /**
   * Sends all groupdata to the server, is they're complete.
   * @param {event} event
   */
  const addGroupClick = (event) => {
    let isPublic = (committedState.dropdown['group-creation-visibility-select']['value'] == 1) ? true : false;
    let [memberIDs, adminIDs] = [[], []];

    /* for creating groups, only send the id of users */
    committedState.group_members.map((member) => memberIDs.push(member.id));
    committedState.group_admins.map((admin) => adminIDs.push(admin.id));

    setCreationStatus(STATUS_CREATING)
    client.addGroup(
      committedState.new_group_name,
      isPublic,
      memberIDs,
      adminIDs
    ).then((group) => {
      // If adding was successful, redirect to rendering of newly created group
      console.log(group.id);

      client.getGroups().then((groups) => {
        console.log(groups);
        navigate(`/group/${group.id}`)
      })
    })
      .catch((error) => {
        setCreationStatus(error.response.status)
        console.log(error.response);
      });
  };

  const showCreationStatus = () => {
    if (creationStatus === STATUS_NONE) {
      return undefined
    } else if (creationStatus === STATUS_CREATING) {
      return <CircularProgress/>
    } else {
      //TODO: Add error message
      return <Alert severity="error"> Error {creationStatus} </Alert>
    }
  }

  /**
   * Gets the logged in user and sets him/her/it to the memberlist.
   */
  const getLoggedUser = () => {
    client.getCurrentUser().then((user) => {
      uncommittedState.group_members.push(user);
      commitState();
    })
      .catch((error) => {
        console.log(error.response);
      })
  };

  /**
   * Adds user to the group as member and avoids duplicate members.
   * Logged user is automatically a member.
   * @param {object} user
   * */
  const addMembers = (user) => {
    let isAlreadyMember = false;

    uncommittedState.group_members.map(member => {
      if (user.id == member.id) {
        isAlreadyMember = true;
      }
    })

    if (!isAlreadyMember) {
      uncommittedState.group_members.push(user);
      commitState();
      isAlreadyMember = false;

    } else {
      console.log("user is already in group_members");
    }
  };

  /**
   * Add admins to the group.
   * @param {object} admin
   */
  const addAdmin = (admin) => {
    let isAlreadyAdmin = false;
    uncommittedState.group_admins.map(member => {
      if (admin.id == member.id) {
        isAlreadyAdmin = true;
      }
    })

    if (!isAlreadyAdmin) {
      uncommittedState.group_admins.push(admin);
      commitState();
      isAlreadyAdmin = false;

    } else {
      console.log("user is already in group_admins");
    }
  };

  /**
   * Checks is logged user is groupowner.
   * @return {boolean} true, if user is groupowner
   */
  const currentUserIsOwner = () => {
    return (client.user.id === state.group.owner)
  }

  /**
   * Checks is logged user is groupadmin.
   * @return {boolean} true, if user is admin
   */
  const currentUserIsAdmin = () => {
    return state.group.group_admins.indexOf(client.user.id) >= 0
  }

  /**
   * Deletes a user from groupmemberlist.
   * @param {object} member
   * @param {string} context
   */
  const deleteUser = (member, context) => {
    console.log("deleted user: ", member.username)
    if (context == MEMBER_CONTEXT) {
      uncommittedState.group_members = uncommittedState.group_members.filter(user => user.username !== member.username);
      committedState.group_members = uncommittedState.group_members;
      commitState();
    } else {
      uncommittedState.group_admins = uncommittedState.group_admins.filter(user => user.username !== member.username);
      committedState.group_admins = uncommittedState.group_admins;
      commitState();
    }
  }

  /**
   * Builds dynamically a memberlist and will be shown in the view.
   * @param {array} list
   * @param {string} context
   * @return {HTMLElement} built memberlist
   */
  const buildUserList = (list, context) => {
    if (list != null) {
      // here is a memberlist
      return (<List>
        {list.map((member) => (
          <ListItem button onClick={() => {
            deleteUser(member, context)
          }}>
            <ListItemAvatar>
              <Avatar>
                <FolderIcon/> {/* TODO: Insert User Avatar here*/}
              </Avatar>
            </ListItemAvatar>
            <ListItemText primaryTypographyProps={{color: "primary"}} primary={member.username}/>
          </ListItem>
        ))}
      </List>)
    }
  }

  /**
   * Callbackfunction that gets all the chosen members from AddUserDialog.
   * @param {object} members
   */
  const handleMemberCallback = (members) => {
    members.map(member => addMembers(member));
  }

  /**
   * Callbackfunction that gets all the chosen admins from AddAdminDialog.
   * @param {object} admins
   */
  const handleAdminCallback = (admins) => {
    admins.map(admin => addAdmin(admin));
  }


  React.useEffect(() => {
    getLoggedUser();
  }, []);


  return (
    <div className={classes.root}>
    <Paper className={classes.paper}>
      <Grid container spacing={3}>
        <Grid item xs={12}>
          <h2>Erstellen Sie eine neue Gruppe</h2>
        </Grid>
        <Grid item xs={3}>
          <Typography className={classes.typography}>Name der Gruppe</Typography>
        </Grid>
        <Grid item xs={9}>
          <TextField
            id='group-creation-new-group-name'
            className={classes.textField}
            label='Gruppenname'
            type='text'
            value={uncommittedState.new_group_name}
            InputProps={{
              className: classes.input,
            }}
            onChange={(event) => {
              uncommittedState.new_group_name = event.target.value;
              commitState();
            }}/>
        </Grid>
        <Grid item xs={3}>
          <Typography className={classes.typography}>Nutzer zur Gruppe hinzufügen</Typography>
        </Grid>
        <Grid item xs={2}>
          <AddUserDialog parentMemberCallback={handleMemberCallback}/>
        </Grid>
        <Grid item xs={7}>
          {buildUserList(committedState.group_members, MEMBER_CONTEXT)}
        </Grid>
        <Grid item xs={3}>
          <Typography className={classes.typography}>Admin zur Gruppe hinzufügen</Typography>
        </Grid>

        <Grid item xs={2}>
          <AddAdminDialog parentAdminCallback={handleAdminCallback}/>
        </Grid>
        <Grid item xs={7}>
          {buildUserList(committedState.group_admins, ADMIN_CONTEXT)}
        </Grid>
        <Grid item xs={3}>
          <Typography className={classes.typography}>Gruppen-Sichtbarkeit</Typography>
        </Grid>
        <Grid item xs={9}>
          <Select
            labelId="group-creation-visibility-select-label"
            id="group-creation-visibility-select"
            className={classes.select}
            open={uncommittedState.dropdown['group-creation-visibility-select']['open']}
            onClose={handleClose}
            onOpen={handleOpen}
            value={uncommittedState.dropdown['group-creation-visibility-select']['value']}
            onChange={handleChange}
          >
            <MenuItem value={0}> Privat </MenuItem>
            <MenuItem value={1}> Öffentlich </MenuItem>;
          </Select>
        </Grid>
        <Grid item xs={10}>
        </Grid>
        <Grid item xs={2}>
          <Button
            onClick={addGroupClick}
            variant="contained"
            component="label"
            color="primary"
            disabled={committedState.new_group_name.length === 0}
          >
            Gruppe erstellen
          </Button>
          { showCreationStatus() }
        </Grid>
      </Grid>
    </Paper>
    </div>
  );
};

export default CreateGroup;	