import React, { useState, useRef, useEffect } from 'react';
import useStateWithCallback from 'use-state-with-callback';

/* Material UI Core */
import { makeStyles } from '@material-ui/core/styles';
import List from '@material-ui/core/List';
import Divider from '@material-ui/core/Divider';
import IconButton from '@material-ui/core/IconButton';
import ListItem from '@material-ui/core/ListItem';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemText from '@material-ui/core/ListItemText';
import ListItemSecAct from '@material-ui/core/ListItemSecondaryAction';
import Collapse from '@material-ui/core/Collapse';
import ExpandLess from '@material-ui/icons/ExpandLess';
import ExpandMore from '@material-ui/icons/ExpandMore';

/* Material UI Icons */
import InboxIcon from '@material-ui/icons/MoveToInbox';
import AccountCircle from "@material-ui/icons/AccountCircle";
import Dashboard from "@material-ui/icons/Dashboard";
import Group from "@material-ui/icons/Group";
import InfoIcon from '@material-ui/icons/Info';
import HelpIcon from '@material-ui/icons/Help';

/* Selfcreated components */
import AddChartMenu from './AddChartMenu';
import AddDashboardMenu from './AddDashboardMenu';
import AddGroupsMenu from './AddGroupsMenu';
import CollapseList from './CollapseList';
import CollapseListDashboard from './CollapseListDashboard';


const DrawerList = (props) => {
  const useStyles = makeStyles((theme) => ({
    nested: {
        paddingLeft: theme.spacing(4),
      },
  }));
  const classes = useStyles();
  const [openCharts, setOpenCharts] = React.useState(false);
  const [changedNLI, setNLI] = React.useState(false);
  const [chart, setChart] = useStateWithCallback("", chart => {
   
  });
  const [openDashboard, setOpenDashboard] = React.useState(false);
  const [dashboard, setDashboard] = React.useState("");
  //const [chart, setChart] = React.useState("");

  const [openGroup, setOpenGroup] = React.useState(false);
  const [group, setGroup] = React.useState("");
  
  let open = props.parentIsDrawerOpen;
  let chartsList = [];
  let collapseProps = {};
  let chartName = "";

  const handleClickCharts = () => {
    setOpenCharts(!openCharts);
  };


  const callbackFunc = (childData) => {
    chartName = childData;
    setChart(childData);
  };



  return (<div>
    <List>
          <ListItem button key="Account">
              <ListItemIcon><AccountCircle /></ListItemIcon>
              <ListItemText primary="Account" />
          </ListItem>

          <ListItem button key="Charts" onClick={handleClickCharts}>
              <ListItemIcon><InboxIcon /></ListItemIcon>
              <ListItemText primary="Charts"/>
              <ListItemSecAct>
                <IconButton edge="end" aria-label="add">
                  {open ? <AddChartMenu plotnameCallback={callbackFunc} /> : null}
                </IconButton>
              </ListItemSecAct>
              {openCharts ? <ExpandLess /> : <ExpandMore />}
           </ListItem>
              {chart.length > 0 ? <CollapseList name={chart} open={openCharts}/> : null}

           <ListItem button key="Dashboards">
              <ListItemIcon><Dashboard /></ListItemIcon>
              <ListItemText primary="Dashboards"/>
              <ListItemSecAct>
                <IconButton edge="end" aria-label="add">
                  {open ? <AddDashboardMenu /> : null}
                </IconButton>
              </ListItemSecAct>
           </ListItem>
              {dashboard.length > 0 ? <CollapseListDashboard name={dashboard} open={openDashboard}/> : null}

           <ListItem button key="Groups">
              <ListItemIcon><Group /></ListItemIcon>
              <ListItemText primary="Groups"/>
              <ListItemSecAct>
                <IconButton edge="end" aria-label="add">
                  {open ? <AddGroupsMenu /> : null}
                </IconButton>
              </ListItemSecAct>
           </ListItem>
              {group.length > 0 ? <CollapseListDashboard name={group} open={openGroup}/> : null}
        </List>
        <Divider />
        <List>
          <ListItem button key="Info">
              <ListItemIcon><InfoIcon /></ListItemIcon>
              <ListItemText primary="Info" />
          </ListItem>
          <ListItem button key="Help">
              <ListItemIcon><HelpIcon /></ListItemIcon>
              <ListItemText primary="Help" />
          </ListItem>
        </List>
  </div>
  );
}
