import React, {useState, useRef} from "react";
import MenuItem from '@material-ui/core/MenuItem';
import {APIContext} from './callWrapper'
import Avatar from '@material-ui/core/Avatar';
import Paper from '@material-ui/core/Paper';
import Typography from "@material-ui/core/Typography";
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import ListItemAvatar from '@material-ui/core/ListItemAvatar';
import ListItemIcon from '@material-ui/core/ListItemIcon';
import ListItemSecondaryAction from '@material-ui/core/ListItemSecondaryAction';
import FolderIcon from '@material-ui/icons/Folder';
import DeleteIcon from '@material-ui/icons/Delete';
import ListItemText from '@material-ui/core/ListItemText';
import {useNavigate} from "react-router-dom";
import Modal from "@material-ui/core/Modal";
import Login from "./Login";
import Toolbar from "@material-ui/core/Toolbar";
import {makeStyles} from "@material-ui/core/styles";
import {Button, TextField} from "@material-ui/core";
import Switch from '@material-ui/core/Switch';
import Grid from '@material-ui/core/Grid';
import TablePagination from '@material-ui/core/TablePagination';

import {useParams} from "react-router-dom";

const drawerWidth = 240;
const useStyles = makeStyles((theme) => ({
  root: {
    display: 'flex',
    'min-height': '100vh',
    position: 'relative',
    'padding-bottom': '60px',
  },
  appBar: {
    zIndex: theme.zIndex.drawer + 1,
    transition: theme.transitions.create(['width', 'margin'], {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.leavingScreen,
    }),
  },
  appBarShift: {
    marginLeft: drawerWidth,
    width: `calc(100% - ${drawerWidth}px)`,
    transition: theme.transitions.create(['width', 'margin'], {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.enteringScreen,
    }),
  },
  menuButton: {
    marginRight: 36,
  },
  hide: {
    display: 'none',
  },
  drawer: {
    width: drawerWidth,
    flexShrink: 0,
    whiteSpace: 'nowrap',
  },
  drawerOpen: {
    width: drawerWidth,
    transition: theme.transitions.create('width', {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.enteringScreen,
    }),
  },
  drawerClose: {
    transition: theme.transitions.create('width', {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.leavingScreen,
    }),
    overflowX: 'hidden',
    width: theme.spacing(7) + 1,
    [theme.breakpoints.up('sm')]: {
      width: theme.spacing(9) + 1,
    },
  },
  toolbar: {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'flex-end',
    padding: theme.spacing(0, 1),
    // necessary for content to be below app bar
    ...theme.mixins.toolbar,
  },
  content: {
    flexGrow: 1,
    padding: theme.spacing(3),
  },
  paper: {
    boxShadow: theme.shadows[5],
    padding: 25,
    margin: 12,
  },
  spacing: {
    marginTop: 40,
  }
}));

const GroupDetails = (props) => {

    const MEMBER_CONTEXT = "memberContext";
    const ADMIN_CONTEXT = "adminContext";
    const CHART_CONTEXT = "chartContext";
    const DATASOURCE_CONTEXT = "datasourceContext";

    const client = React.useContext(APIContext)
    const classes = useStyles();
    const history = useNavigate()
    const [state, setState] = React.useState({group: null, owner: null, members: null, admins: null})
    const [deleteModalConfig, setDeleteModalConfig] = React.useState({open: false, obj: null, context: null})
    const [UIState, setUIState] = React.useState({
      "user-add-search": "",
      "admin-switch": false,
      "userSearchPagination": {"page": 0, "rowsPerPage": 10},
      [MEMBER_CONTEXT]: {"page": 0, "rowsPerPage": 10},
      [ADMIN_CONTEXT]: {"page": 0, "rowsPerPage": 10}
    })
    const [userSearchQueueIndex, setUserSearchQueueIndex] = React.useState(null)
    const [userSearchList, setUserSearchList] = React.useState([])

    const closeModal = () => {
      setDeleteModalConfig({open: false, obj: null, context: null})
    }

    const openModal = (obj, context) => {
      setDeleteModalConfig({open: true, obj: obj, context: context})
    }

    const updateGroup = (force) => {
      if (force || state["group"] == null) {
        client.getGroup(props.groupID).then(async (updated_group) => {
          let newState = {}
          newState["group"] = updated_group
          newState["owner"] = await client.getUser(updated_group.owner)
          newState["members"] = await client.getUsers(updated_group.group_members)
          newState["admins"] = await client.getUsers(updated_group.group_admins)
          setState(newState)
        }).catch((error) => {
          //TODO: Error handling, determine which error to show
          console.error(error)
        });
      }
    }

    const setUserSearchField = (newContent) => {
      let uncommittedState = JSON.parse(JSON.stringify(UIState));
      uncommittedState["user-add-search"] = newContent;
      setUIState(uncommittedState)
      if (userSearchQueueIndex != null) {
        clearTimeout(userSearchQueueIndex)
      }
      setUserSearchQueueIndex(setTimeout(() => {
        setUserSearchQueueIndex(null);
        client.getUserByName(newContent).then((matchingUsers) => {
          setUserSearchList(matchingUsers)
        })
      }, 500))
    }

    const setPagination = (page, rowsPerPage, paginationID) => {
      let uncommittedState = JSON.parse(JSON.stringify(UIState));
      uncommittedState[paginationID]["page"] = page
      uncommittedState[paginationID]["rowsPerPage"] = rowsPerPage
      setUIState(uncommittedState)
    }

    const buildDialog = (obj, context) => {
      let type = "";
      let name = "";
      switch (context) {
        case MEMBER_CONTEXT:
          type = "Mitglied ";
          name = obj.username;
          break;
        case ADMIN_CONTEXT:
          type = "Administrator ";
          name = obj.username;
          break;
        case CHART_CONTEXT:
          type = "Chart ";
          name = obj.chart_name;
          break;
        case DATASOURCE_CONTEXT:
          type = "Datenquelle ";
          name = obj.datasource_name;
          break;
        default:
          break;
      }
      return `${type} ${name} wirklich aus Gruppe entfernen?`
    }

    const deleteModal = (<Modal
      open={deleteModalConfig.open}
      onClose={closeModal}
      aria-labelledby="simple-modal-title"
      aria-describedby="simple-modal-description"
    >
      <div style={{top: '30%', left: '30%',}} className={classes.paper}>
        <Typography component="h2" variant="h6" color="primary"
                    gutterBottom> {buildDialog(deleteModalConfig.obj, deleteModalConfig.context)} </Typography>
        <Button variant="contained" component="label" color="primary" onClick={() => {
          let members = [];
          let admins = [];
          if (deleteModalConfig.context === MEMBER_CONTEXT) {
            members[0] = deleteModalConfig.obj.id
          } else if (deleteModalConfig.context === ADMIN_CONTEXT) {
            admins[0] = deleteModalConfig.obj.id
          } else {
            //TODO: Implement delete for charts and datasources.
            throw "UNSUPPORTED DELETE OPERATION"
          }
          client.removeGroupMembers(state.group.id, admins, members).then(() => {
            updateGroup(true)
          }).catch((error) => {
            //TODO: Inform user about error
            console.error(error)
            closeModal();
          })
          closeModal();
        }
        }> Löschen</Button>
        <Button variant="contained" component="label" color="primary" onClick={() => {
          closeModal();
        }
        }> Abbrechen </Button>
      </div>
    </Modal>)

    const currentUserIsOwner = () => {
      return (client.user.id === state.group.owner)
    }

    const currentUserIsAdmin = () => {
      return state.group.group_admins.indexOf(client.user.id) >= 0
    }

    React.useEffect(() => {
      console.log("State changed");
      updateGroup();
    })

    React.useEffect(() => {
      //Run only once on mount
    }, []);

    const buildUserList = (list, context) => {
      const slice = list.slice(UIState[context]["page"] * UIState[context]["rowsPerPage"], (UIState[context]["page"] + 1) * UIState[context]["rowsPerPage"])
      return (<List>
        {slice.map((member) => (
          <ListItem>
            <ListItemAvatar>
              <Avatar>
                <FolderIcon/> {/* TODO: Insert User Avatar here*/}
              </Avatar>
            </ListItemAvatar>
            <ListItemText primaryTypographyProps={{color: "primary"}} primary={member.username}/>
            {
              (currentUserIsOwner() || (currentUserIsAdmin())) ?
                (<DeleteIcon variant="contained" color="secondary" onClick={() => {
                  openModal(member, context)
                }
                }/>) : undefined
            }
          </ListItem>
        ))}
      </List>)
    }

    return (
      (state["group"] != null && client.user != null) ? ( /* Assert that a group is loaded and user is logged in, otherwise return an empty Fragment*/
        <div>
          <Paper className={classes.paper}>
            {deleteModal}
            <h1> {state["group"].name} </h1>
            <h3> {"Ersteller dieser Gruppe ist " + state["owner"].username}</h3>
            <h2 className={classes.spacing}> Mitglieder </h2>
            {buildUserList(state["members"], MEMBER_CONTEXT)}
            <TablePagination count={state["members"].length}
                             rowsPerPage={UIState[MEMBER_CONTEXT]["rowsPerPage"]}
                             page={UIState[MEMBER_CONTEXT]["page"]}
                             onChangePage={(event) => {
                               setPagination(event.target.value, UIState[MEMBER_CONTEXT]["rowsPerPage"], MEMBER_CONTEXT)
                             }}
                             onChangeRowsPerPage={(event) => {
                               setPagination(UIState[MEMBER_CONTEXT]["page"], event.target.value, MEMBER_CONTEXT)
                             }}
            />
            <h2 className={classes.spacing}> Administratoren </h2>
            {buildUserList(state["admins"], ADMIN_CONTEXT)}
            <TablePagination count={state["admins"].length}
                             rowsPerPage={UIState[ADMIN_CONTEXT]["rowsPerPage"]}
                             page={UIState[ADMIN_CONTEXT]["page"]}
                             onChangePage={(event) => {
                               setPagination(event.target.value, UIState[ADMIN_CONTEXT]["rowsPerPage"], ADMIN_CONTEXT)
                             }}
                             onChangeRowsPerPage={(event) => {
                               setPagination(UIState[ADMIN_CONTEXT]["page"], event.target.value, ADMIN_CONTEXT)
                             }}
            />
          </Paper>
          {(currentUserIsOwner() || currentUserIsAdmin()) ? (
            <Paper className={classes.paper}>
              {deleteModal}
              <h2> Benutzer hinzufügen</h2>
              <TextField id='user-add-search' label='Benutzername' type='text'
                         value={UIState["user-add-search"]} onChange={(event) => {
                setUserSearchField(event.target.value)
              }}/>


              <List>
                {userSearchList.slice(UIState["userSearchPagination"]["page"] * UIState["userSearchPagination"]["rowsPerPage"], (UIState["userSearchPagination"]["page"] + 1) * UIState["userSearchPagination"]["rowsPerPage"]).map((member) => (
                  <ListItem>
                    <ListItemAvatar>
                      <Avatar>
                        <FolderIcon/> {/* TODO: Insert User Avatar here*/}
                      </Avatar>
                    </ListItemAvatar>
                    <ListItemText primaryTypographyProps={{color: "primary"}} primary={member.username}/>
                    {/* TODO: Adding a user scrolls the page, making misclicks easy. Get absolute position from bottom of page and scroll back to that on redraw */}
                    <Button variant="contained" component="label" color="primary" onClick={() => {
                      client.addGroupMembers(state.group.id, [], [member.id]).then(() => updateGroup(true))
                    }
                    }> + Mitglied </Button>
                    <Button variant="contained" component="label" color="primary" onClick={() => {
                      client.addGroupMembers(state.group.id, [member.id], []).then(() => updateGroup(true))
                    }
                    }> + Admin </Button>
                  </ListItem>
                ))}
              </List>
              <TablePagination count={userSearchList.length}
                               rowsPerPage={UIState["userSearchPagination"]["rowsPerPage"]}
                               page={UIState["userSearchPagination"]["page"]}
                               onChangePage={(event) => {
                                 setPagination(event.target.value, UIState["userSearchPagination"]["rowsPerPage"], "userSearchPagination")
                               }}
                               onChangeRowsPerPage={(event) => {
                                 setPagination(UIState["userSearchPagination"]["page"], event.target.value, "userSearchPagination")
                               }}
              />
            </Paper>) : undefined
          }
        </div>
      ) : (<React.Fragment></React.Fragment>)
    );
  }
;

export function GroupRoute({dimensions}) {
    const params = useParams()

    return <GroupDetails groupID={ params["groupID"] } />
}

export default GroupDetails;
