import React from 'react';

import TextField from "@material-ui/core/TextField";
import Button from "@material-ui/core/Button";

import {APIContext} from "./callWrapper";
import {Typography} from "@material-ui/core";


export default class Login extends React.Component {

    constructor(props) {
        super(props);
        this.onSuccess = props.onSuccess;
        this.state = {
            username: '',
            password: '',
            lastError: undefined,
        };
    }

    renderErrors() {
        //TODO: Styling
        if (this.state.lastError) {
            if (this.state.lastError.response !== undefined) {
                if (500 <= this.state.lastError.response.status && this.state.lastError.response.status <= 599) {
                    //Server Error, currently unavailable, try again later
                    return <Typography variant='body1' color='primary'> Es ist ein Fehler aufgetreten, bitte versuchen sie es später
                        erneut! </Typography>
                } else if (400 <= this.state.lastError.response.status && this.state.lastError.response.status <= 499) {
                    // Client error, most likely wrong user/pass
                    return <Typography variant='body1' color='primary'> Benutzername oder Passwort falsch! </Typography>
                }
            } else {
                // No connection established. Could be a network error. Probably unrecoverable
                return <Typography variant='body1' color='primary'> Es konnte keine Verbindung zum Server aufgebaut werden. </Typography>
            }
        } else {
            return null
        }
    }

    render() {
        return (
            <APIContext.Consumer>
                {(client) => (
                    <div>
                        { this.renderErrors() }
                        <TextField id='login-username' label='Username' value={this.state.username} onChange={(event) => {
                            this.state.username = event.target.value;
                            this.setState(this.state)
                        }}/> <br/>
                        <TextField id='password' label='Password' type='password' value={this.state.password}
                                   onChange={(event) => {
                                       this.state.password = event.target.value;
                                       this.setState(this.state)
                                   }}/><br/>
                        <Button id='login-button' onClick={() => {
                            if (this.state.password1 === this.state.password2) {
                                client.login(this.state.username, this.state.password).then(() => {
                                    //TODO: Tell user to verify email and login
                                    //Login attempts should fail until verified
                                    this.onSuccess()
                                    console.log("Logged in...")
                                }).catch((error) => {
                                        this.state.lastError = error;
                                        this.setState(this.state)
                                    }
                                )
                            } else {
                                this.state.lastError = "Passwörter stimmen nicht überein";
                                this.setState(this.state)
                            }
                        }}> Login </Button>
                    </div>
                )}
            </APIContext.Consumer>
        )
    }
}