import React, {useState, useRef} from "react";
import {APIContext} from './callWrapper'
import Paper from '@material-ui/core/Paper';
import Typography from "@material-ui/core/Typography";
import {useNavigate} from "react-router-dom";
import {Button, FormHelperText, Input, InputLabel, TextField} from "@material-ui/core";
import Switch from '@material-ui/core/Switch';
import Grid from '@material-ui/core/Grid';
import FormControl from "@material-ui/core/FormControl";
import { makeStyles } from '@material-ui/core/styles';

const useStyles = makeStyles((theme) => ({
  root: {
    flexGrow: 1,
  },
  divider: {
    margin: theme.spacing(2, 0),
  },
  textField: {
    borderColor: 'white'
    },
    "& .MuiFormLabel-root": {
      color: "white"
    },
    input: {
        color: 'white'
    },
  select: {
    "&:before": {
      borderBottomColor: "white",
    },"&:after": {
      borderBottomColor: "white",
    },
    "& .MuiSvgIcon-root": {
      color: "white",
    },
    "color": "white",
  },
  paper: {
    padding: 25,
    margin: 12
  },
  switchLabel: {
    width: 180,
    top: 16
  },
  textfieldLabel: {
    top: -38,
  },
  button: {
    right: 60,
    position: 'absolute',
    color: 'white',
    backgroundColor: '#6398CC',
    "&:hover, &:focus": {
      backgroundColor: '#335D94',
    },
    "&:active": {
      backgroundColor: '#0F3F7B',
    },
  },
  typography: {
    color: 'black',
    'font-weight': 500,
  },
  headerName: {
    marginBottom: 14,
  }
}));

const ProfilePageView = () => {

    const client = React.useContext(APIContext)
    const classes = useStyles();
    const history = useNavigate()
    const [UIState, setUIState] = React.useState({
      "display-name-field": "",
      "first-name-field": "",
      "last-name-field": "",
      "show-real-name-switch": false,
      "searchability-switch": false,
      "password0-field": "",
      "password1-field": "",
      "password2-field": "",
      "passwordCurrent-field": "",
      "mail0-field": "",
      "mail1-field": "",
      "state_loaded": false,
      "last_error": undefined,
    })

    const setUIElement = (event) => {
      let uncommittedState = JSON.parse(JSON.stringify(UIState));
      if (event.target.type === "text" || event.target.type === "password") {
        uncommittedState[event.target.id] = event.target.value
      } else if (event.target.type === "checkbox") {
        uncommittedState[event.target.id] = event.target.checked
      }

      setUIState(uncommittedState)
    }

    const initFromServer = () => {

      /* Initialise all fields from the server state
       Only allow uploading changes, when previously initialised to avoid versioning conflicts */
      client.getCurrentUser().then((user) => {
        let uncommittedState = JSON.parse(JSON.stringify(UIState));
        uncommittedState["first-name-field"] = user["first_name"]
        uncommittedState["last-name-field"] = user["last_name"]
        uncommittedState["display-name-field"] = user["username"]
        uncommittedState["searchability-switch"] = user["public_profile"]
        uncommittedState["show-real-name-switch"] = user["real_name"]
        uncommittedState["state_loaded"] = true
        setUIState(uncommittedState)
      }).catch((error) => {
        /* Server state could not be acquired, disable upload */
        //TODO: Inform user of connectivity problems
        let uncommittedState = JSON.parse(JSON.stringify(UIState));
        uncommittedState["state_loaded"] = false
        setUIState(uncommittedState)
      })
    }

    const changeUserData = () => {
      client.updateCurrentUser(
        UIState["first-name-field"],
        UIState["last-name-field"],
        UIState["display-name-field"],
        UIState["searchability-switch"],
        UIState["show-real-name-switch"],
      ).then(console.log)
    }

    const changePassword = () => {
      if (UIState["password1-field1"] !== UIState["password1-field2"]) {
        //TODO: Set Error passwords not identical
      } else {
        client.changePassword(UIState["password0-field"], UIState["password1-field"]).then( () => {
          //TODO: Password changed, notify user
          console.log("Password changed successfully")
        }).catch( (error) => {
          console.error(error)
        })
      }
    }

    const changeEmail = () => {
      if (UIState["mail0-field"] !== UIState["mail1-field"]) {
        //TODO: Set Error passwords not identical
      } else {
        client.changeEmail(UIState["passwordCurrent-field"], UIState["mail0-field"]).then( () => {
          //TODO: Password changed, notify user
          console.log("Password changed successfully")
        }).catch( (error) => {
          console.error(error)
        })
      }
    }

    React.useEffect(() => {
      //
      if (!UIState["state_loaded"] && client.user != null) {
        initFromServer()
      }
    })

    React.useEffect(() => {
      //Run only once on mount
    }, []);

    return (
      <React.Fragment>
        <div>
          <Paper className={classes.paper}>
            <Grid container spacing={3}>
              <Grid item xs={12} className={classes.headerName}>
                <Typography className={classes.typography}>Name</Typography>
              </Grid>
              <Grid item xs={3}>
                <FormControl>
                  <InputLabel className={classes.textfieldLabel} htmlFor="display-name-field">Anzeigename</InputLabel>
                  <TextField id="display-name-field" aria-describedby="display-name-field-helper"
                             value={UIState["display-name-field"]} onChange={setUIElement}/>
                  <FormHelperText id="display-name-field-helper"> </FormHelperText>
                </FormControl>
              </Grid>
              <Grid item xs={3}>
                <FormControl>
                  <InputLabel className={classes.textfieldLabel} htmlFor="first-name-field">Vorname</InputLabel>
                  <TextField id="first-name-field" aria-describedby="first-name-field-helper"
                             value={UIState["first-name-field"]} onChange={setUIElement}/>
                  <FormHelperText id="first-name-field-helper"> </FormHelperText>
                </FormControl>
              </Grid>
              <Grid item xs={3}>
                <FormControl>
                  <InputLabel className={classes.textfieldLabel} htmlFor="last-name-field">Nachname</InputLabel>
                  <TextField id="last-name-field" aria-describedby="last-name-field-helper"
                             value={UIState["last-name-field"]} onChange={setUIElement}/>
                  <FormHelperText id="last-name-field-helper"> </FormHelperText>
                </FormControl>
              </Grid>
              <Grid item xs={3}>
              </Grid>
              <Grid item xs={3}>
                <FormControl>
                  <InputLabel className={classes.switchLabel} htmlFor="show-real-name-switch">Echten Namen anzeigen</InputLabel>
                  <Switch id="show-real-name-switch" aria-describedby="show-real-name-switch-helper"
                          checked={UIState["show-real-name-switch"]} onChange={setUIElement}/>
                  <FormHelperText id="show-real-name-switch-helper"> </FormHelperText>
                </FormControl>
              </Grid>
              <Grid item xs={3}>
                <FormControl>
                  <InputLabel className={classes.switchLabel} htmlFor="searchability-switch">Profil suchbar machen</InputLabel>
                  <Switch id="searchability-switch" aria-describedby="searchability-switch-helper"
                          checked={UIState["searchability-switch"]} onChange={setUIElement}/>
                  <FormHelperText id="searchability-switch-helper"> </FormHelperText>
                </FormControl>
              </Grid>
              <Grid item xs={3}>
              </Grid>
              {UIState["state_loaded"] ? (<Grid item xs={3}>
                <Button 
                  className={classes.button} 
                  id="settings-submit" 
                  onClick={changeUserData}>
                    Änderungen speichern
                </Button>
              </Grid>) : null}
            </Grid>
          </Paper>
        </div>
        <div>
          <Paper className={classes.paper}>
            <Grid container spacing={3}>
              <Grid item xs={12}>
                <Typography className={classes.typography}> Passwort </Typography>
              </Grid>
              <Grid item xs={3}>
                <InputLabel htmlFor="password0-field"> Altes Passwort </InputLabel>
                <TextField type="password" id="password0-field" aria-describedby="password0-field-helper"
                           onChange={setUIElement}/>
                <FormHelperText id="password0-field-helper"> </FormHelperText>
              </Grid>
              <Grid item xs={3}>
                <InputLabel htmlFor="password1-field"> Neues Passwort </InputLabel>
                <TextField type="password" id="password1-field" aria-describedby="password1-field-helper"
                           onChange={setUIElement}/>
                <FormHelperText id="password1-field-helper"> </FormHelperText>
              </Grid>
              <Grid item xs={3}>
                <InputLabel htmlFor="password2-field"> Passwort erneut eingeben </InputLabel>
                <TextField type="password" id="password2-field" aria-describedby="password2-field-helper"
                           onChange={setUIElement}/>
                <FormHelperText id="password2-field-helper"> </FormHelperText>
              </Grid>
              <Grid item xs={3}>
                <Button 
                  id="password-submit" 
                  className={classes.button} 
                  onClick={changePassword}> 
                    Passwort ändern
                </Button>
              </Grid>
            </Grid>
          </Paper>

          <Paper className={classes.paper}>
            <Grid container spacing={3}>
              <Grid item xs={12}>
                <Typography className={classes.typography}> E-Mail </Typography>
              </Grid>
              <Grid item xs={3}>
                <InputLabel htmlFor="passwordCurrent-field"> Aktuelles Passwort </InputLabel>
                <TextField type="password" id="passwordCurrent-field" aria-describedby="passwordCurrent-field-helper"
                           onChange={setUIElement}/>
                <FormHelperText id="passwordCurrent-field-helper"> </FormHelperText>
              </Grid>
              <Grid item xs={3}>
                <InputLabel htmlFor="mail0-field"> Neue E-Mailadresse </InputLabel>
                <TextField type="text" id="mail0-field" aria-describedby="mail0-field-helper"
                           onChange={setUIElement}/>
                <FormHelperText id="mail0-field-helper"> </FormHelperText>
              </Grid>
              <Grid item xs={3}>
                <InputLabel htmlFor="mail1-field"> Neue E-Mailadresse erneut eingeben </InputLabel>
                <TextField type="text" id="mail1-field" aria-describedby="mail1-field-helper"
                           onChange={setUIElement}/>
                <FormHelperText id="mail1-field-helper"> </FormHelperText>
              </Grid>
              <Grid item xs={3}>
                <Button 
                  id="email-submit" 
                  className={classes.button} 
                  onClick={changeEmail}>
                     E-Mail ändern 
                </Button>
              </Grid>
            </Grid>
          </Paper>
        </div>
      </React.Fragment>
    );
  }
;

export default ProfilePageView;
