import React from 'react';

import TextField from "@material-ui/core/TextField";
import Button from "@material-ui/core/Button";

import {APIContext} from "./callWrapper";
import {Typography} from "@material-ui/core";


export default class Register extends React.Component {

    constructor(props) {
        super(props);
        this.onSuccess = props.onSuccess;
        this.state = {
            username: '',
            password1: '',
            password2: '',
            email: '',
            lastError: undefined,
        };
    }

    renderErrors() {
        //TODO: Styling
        if (this.state.lastError) {
            if (this.state.lastError.response !== undefined) {
                if (500 <= this.state.lastError.response.status && this.state.lastError.response.status <= 599) {
                    //Server Error, currently unavailable, try again later
                    return <Typography variant='body1' color='primary'> Es ist ein Fehler aufgetreten, bitte versuchen sie es später
                        erneut! </Typography>
                } else if (400 <= this.state.lastError.response.status && this.state.lastError.response.status <= 499) {
                    // Client error, most likely wrong user/pass
                    return <Typography variant='body1' color='primary'> {this.state.lastError.request.responseText} </Typography>
                }
            } else {
                // No connection established. Could be a network error. Probably unrecoverable
                return <Typography variant='body1' color='primary'> Es konnte keine Verbindung zum Server aufgebaut werden. </Typography>
            }
        } else {
            return null
        }
    }

    render() {
        return (
            <APIContext.Consumer>
                {(client) => (
                    <div>
                        { this.renderErrors() }
                        <TextField id='email' label='E-Mail' value={this.state.email} onChange={(event) => {
                            this.state.email = event.target.value;
                            this.setState(this.state)
                        }}/> <br/>
                        <TextField id='register-username' label='Username' value={this.state.username} onChange={(event) => {
                            this.state.username = event.target.value;
                            this.setState(this.state)
                        }}/> <br/>
                        <TextField id='password1' label='Password' type='password' value={this.state.password1}
                                   onChange={(event) => {
                                       this.state.password1 = event.target.value;
                                       this.setState(this.state)
                                   }}/><br/>
                        <TextField id='password2' label='Password erneut eingeben' type='password' value={this.state.password2}
                                   onChange={(event) => {
                                       this.state.password2 = event.target.value;
                                       this.setState(this.state)
                                   }}/><br/>
                        <Button id='register-button' onClick={() => {
                            client.addUser(this.state.email, this.state.password1, this.state.username).then(this.onSuccess).catch((error) => {
                                    this.state.lastError = error;
                                    this.setState(this.state)
                                }
                            )
                        }}> Jetzt registrieren </Button>
                    </div>
                )}
            </APIContext.Consumer>
        )
    }
}