import React from "react";
import {APIContext} from "./callWrapper";
import Typography from "@material-ui/core/Typography";
import TablePagination from "@material-ui/core/TablePagination";
import List from "@material-ui/core/List";
import ListItem from "@material-ui/core/ListItem";
import ListItemAvatar from "@material-ui/core/ListItemAvatar";
import Avatar from "@material-ui/core/Avatar";
import FolderIcon from "@material-ui/icons/Folder";
import ListItemText from "@material-ui/core/ListItemText";
import DeleteIcon from "@material-ui/icons/Delete";
import {Button, TextField} from "@material-ui/core";

const ShareDialog = ({ressourceType, ressourceID}) => {

  const USER_CONTEXT = "memberContext";
  const GROUP_CONTEXT = "adminContext";

  const client = React.useContext(APIContext)

  const [UIState, setUIState] = React.useState({
    "user-add-search": "",
    "admin-switch": false,
    "userSearchPagination": {"page": 0, "rowsPerPage": 10},
    "groupSearchPagination": {"page": 0, "rowsPerPage": 10},
    [USER_CONTEXT]: {"page": 0, "rowsPerPage": 10},
    [GROUP_CONTEXT]: {"page": 0, "rowsPerPage": 10}
  })
  const [userSearchQueueIndex, setUserSearchQueueIndex] = React.useState(null)
  const [availableGroups, setAvailableGroups] = React.useState(null)
  const [userSearchList, setUserSearchList] = React.useState([])
  const [state, setState] = React.useState({groups: null, users: null})

  const updateObject = (force) => {
    if (force || state["groups"] == null) {
      let promise;
      if (ressourceType === "dashboard") {
        promise = client.getDashboardShares(ressourceID)
      } else if (ressourceType === "chart") {
        promise = client.getChartShares(ressourceID)
      } else if (ressourceType === "datasource") {
        promise = client.getDatasourceShares(ressourceID)
      }
      promise.then(async (shares) => {
        let newState = {}
        newState["groups"] = await client.getMultipleGroups(shares.groups)
        newState["users"] = await client.getUsers(shares.users)
        setState(newState)
      }).catch((error) => {
        //TODO: Error handling, determine which error to show
        console.error(error)
      });
      client.getGroups().then((groups) => {
        setAvailableGroups(groups)
      })
    }
  }

  const onRemoveUser = (userID) => {
    let promise;
    if (ressourceType === "dashboard") {
      promise = client.removeDashboardShare(ressourceID, [userID], [])
    } else if (ressourceType === "chart") {
      promise = client.removeChartShare(ressourceID, [userID], [])
    } else if (ressourceType === "datasource") {
      promise = client.removeDashboardShare(ressourceID, [userID], [])
    }
    promise.then(() => {
      updateObject(true)
    })
  }

  const onAddUser = (userID) => {
    let promise;
    if (ressourceType === "dashboard") {
      promise = client.addDashboardShare(ressourceID, [userID], [])
    } else if (ressourceType === "chart") {
      promise = client.addChartShare(ressourceID, [userID], [])
    } else if (ressourceType === "datasource") {
      promise = client.addDatasourceShare(ressourceID, [userID], [])
    }
    promise.then(() => {
      updateObject(true)
    })
  }

  const onRemoveGroup = (groupID) => {
    let promise;
    if (ressourceType === "dashboard") {
      promise = client.removeDashboardShare(ressourceID, [], [groupID])
    } else if (ressourceType === "chart") {
      promise = client.removeChartShare(ressourceID, [], [groupID])
    } else if (ressourceType === "datasource") {
      promise = client.removeDashboardShare(ressourceID, [], [groupID])
    }
    promise.then(() => {
      updateObject(true)
    })
  }

  const onAddGroup = (groupID) => {
    let promise;
    if (ressourceType === "dashboard") {
      promise = client.addDashboardShare(ressourceID, [], [groupID])
    } else if (ressourceType === "chart") {
      promise = client.addChartShare(ressourceID, [], [groupID])
    } else if (ressourceType === "datasource") {
      promise = client.addDatasourceShare(ressourceID, [], [groupID])
    }
    promise.then(() => {
      updateObject(true)
    })
  }

  const setUserSearchField = (newContent) => {
    let uncommittedState = JSON.parse(JSON.stringify(UIState));
    uncommittedState["user-add-search"] = newContent;
    setUIState(uncommittedState)
    if (userSearchQueueIndex != null) {
      clearTimeout(userSearchQueueIndex)
    }
    setUserSearchQueueIndex(setTimeout(() => {
      setUserSearchQueueIndex(null);
      client.getUserByName(newContent).then((matchingUsers) => {
        setUserSearchList(matchingUsers)
      })
    }, 500))
  }

  const setPagination = (page, rowsPerPage, paginationID) => {
    let uncommittedState = JSON.parse(JSON.stringify(UIState));
    uncommittedState[paginationID]["page"] = page
    uncommittedState[paginationID]["rowsPerPage"] = rowsPerPage
    setUIState(uncommittedState)
  }

  const buildList = (list, context) => {
    const slice = list.slice(UIState[context]["page"] * UIState[context]["rowsPerPage"], (UIState[context]["page"] + 1) * UIState[context]["rowsPerPage"])
    return (<List>
      {slice.map((member) => (
        <ListItem>
          <ListItemAvatar>
            <Avatar>
              <FolderIcon/> {/* TODO: Insert User Avatar here*/}
            </Avatar>
          </ListItemAvatar>
          <ListItemText primaryTypographyProps={{color: "primary"}}
                        primary={context === USER_CONTEXT ? member.username : member.name}/>
          {
            <DeleteIcon variant="contained" color="secondary" onClick={() => {
              if (context === USER_CONTEXT) {
                onRemoveUser(member.id)
              } else {
                onRemoveGroup(member.id)
              }
            }}/>
          }
        </ListItem>
      ))}
    </List>)
  }

  React.useEffect(() => {
    updateObject()
  }, []);

  return (state["groups"] != null && client.user != null) ? (<React.Fragment>
    <Typography component="h6" variant="h6" color="primary" gutterBottom> User </Typography>
    {buildList(state["users"], USER_CONTEXT)}
    <TablePagination count={state["users"].length}
                     rowsPerPage={UIState[USER_CONTEXT]["rowsPerPage"]}
                     page={UIState[USER_CONTEXT]["page"]}
                     onChangePage={(event) => {
                       setPagination(event.target.value, UIState[USER_CONTEXT]["rowsPerPage"], USER_CONTEXT)
                     }}
                     onChangeRowsPerPage={(event) => {
                       setPagination(UIState[USER_CONTEXT]["page"], event.target.value, USER_CONTEXT)
                     }}
    />
    <Typography component="h6" variant="h6" color="primary" gutterBottom> User hinzufügen</Typography>
    <TextField id='user-add-search' label='Benutzername' type='text'
               value={UIState["user-add-search"]} onChange={(event) => {
      setUserSearchField(event.target.value)
    }}/>
    <List>
      {userSearchList.slice(UIState["userSearchPagination"]["page"] * UIState["userSearchPagination"]["rowsPerPage"], (UIState["userSearchPagination"]["page"] + 1) * UIState["userSearchPagination"]["rowsPerPage"]).map((member) => (
        <ListItem>
          <ListItemAvatar>
            <Avatar>
              <FolderIcon/> {/* TODO: Insert User Avatar here*/}
            </Avatar>
          </ListItemAvatar>
          <ListItemText primaryTypographyProps={{color: "primary"}} primary={member.username}/>
          <Button variant="contained" component="label" color="primary" onClick={() => {
            onAddUser(member.id)
          }
          }> + User </Button>
        </ListItem>
      ))}
    </List>
    <TablePagination count={userSearchList.length}
                     rowsPerPage={UIState["userSearchPagination"]["rowsPerPage"]}
                     page={UIState["userSearchPagination"]["page"]}
                     onChangePage={(event) => {
                       setPagination(event.target.value, UIState["userSearchPagination"]["rowsPerPage"], "userSearchPagination")
                     }}
                     onChangeRowsPerPage={(event) => {
                       setPagination(UIState["userSearchPagination"]["page"], event.target.value, "userSearchPagination")
                     }}
    />

    <Typography component="h6" variant="h6" color="primary" gutterBottom> Groups </Typography>
    {buildList(state["groups"], GROUP_CONTEXT)}
    <TablePagination count={state["groups"].length}
                     rowsPerPage={UIState[GROUP_CONTEXT]["rowsPerPage"]}
                     page={UIState[GROUP_CONTEXT]["page"]}
                     onChangePage={(event) => {
                       setPagination(event.target.value, UIState[GROUP_CONTEXT]["rowsPerPage"], GROUP_CONTEXT)
                     }}
                     onChangeRowsPerPage={(event) => {
                       setPagination(UIState[GROUP_CONTEXT]["page"], event.target.value, GROUP_CONTEXT)
                     }}
    />
    <Typography component="h6" variant="h6" color="primary" gutterBottom> Verfügbare Gruppen </Typography>
    <List>
      {availableGroups.slice(UIState["groupSearchPagination"]["page"] * UIState["groupSearchPagination"]["rowsPerPage"], (UIState["groupSearchPagination"]["page"] + 1) * UIState["groupSearchPagination"]["rowsPerPage"]).map((member) => (
        <ListItem>
          <ListItemAvatar>
            <Avatar>
              <FolderIcon/> {/* TODO: Insert User Avatar here*/}
            </Avatar>
          </ListItemAvatar>
          <ListItemText primaryTypographyProps={{color: "primary"}} primary={member.name}/>
          <Button variant="contained" component="label" color="primary" onClick={() => {
            onAddGroup(member.id)
          }
          }> + Group </Button>
        </ListItem>
      ))}
    </List>
    <TablePagination count={availableGroups.length}
                     rowsPerPage={UIState["groupSearchPagination"]["rowsPerPage"]}
                     page={UIState["groupSearchPagination"]["page"]}
                     onChangePage={(event) => {
                       setPagination(event.target.value, UIState["groupSearchPagination"]["rowsPerPage"], "groupSearchPagination")
                     }}
                     onChangeRowsPerPage={(event) => {
                       setPagination(UIState["groupSearchPagination"]["page"], event.target.value, "groupSearchPagination")
                     }}
    />
  </React.Fragment>) : <div/>
}

export default ShareDialog