import React, {createRef, useEffect, useState} from 'react';
import './MainContent.css';
import SvgIcon from '@mui/material/SvgIcon';
import TreeView from '@mui/lab/TreeView';
import TreeItem, {treeItemClasses} from '@mui/lab/TreeItem';
import IconButton from "@material-ui/core/IconButton";
import {HorizontalSplit, VerticalSplit} from "@material-ui/icons";
import AddBoxIcon from '@material-ui/icons/AddBox';
import RemoveIcon from '@material-ui/icons/Remove';
import SaveIcon from '@material-ui/icons/Save';
import {APIContext} from "./callWrapper";
import MenuItem from "@material-ui/core/MenuItem";
import Select from "@material-ui/core/Select";
import {TextField} from "@material-ui/core";

export const TreeList = ({tree, setTree, onSave}) => {

  const treePathSeparator = ","
  const client = React.useContext(APIContext)

  const [expanded, setExpanded] = useState([]);
  const [selected, setSelected] = useState([]);
  const [charts, setCharts] = useState({})
  const [selectedChart, setSelectedChart] = useState("");
  const [xAspect, setXAspect] = useState(100)
  const [yAspect, setYAspect] = useState(100)

  const getCharts = () => {
    client.getCharts().then((downloadedCharts) => {
      let newCharts = {}
      for (let chart of downloadedCharts) {
        newCharts[chart.id] = chart
      }
      setCharts(newCharts)
    })
  }

  const updateExpanded = (newExpanded) => {
    if (JSON.stringify(newExpanded) !== JSON.stringify(expanded)) {
      setExpanded(newExpanded)
    }
  }

  const getTreeNode = () => {
    let treeNode = tree
    if (selected[0]) {
      const pathThroughTree = selected[0].split(treePathSeparator)
      for (let pathElement of pathThroughTree) {
        treeNode = treeNode[pathElement]
      }
      return treeNode
    } else {
      return undefined
    }
  }

  const split = (horizontal) => {
    let treeNode = getTreeNode()
    if (treeNode !== undefined) {
      treeNode["split"] = {
        'horizontal': horizontal,
        'aspect': [xAspect, yAspect],
        'c1': {},
        'c2': {},
      }
      setTree(Object.assign({}, tree))
    }
  }

  const verticalSplit = (event) => {
    split(false)
  }

  const horizontalSplit = (event) => {
    split(true)
  }

  const addChart = (event) => {
    let treeNode = getTreeNode()
    if (treeNode !== undefined && selectedChart !== "") {
      treeNode["generatorName"] = "chart"
      treeNode["args"] = {chartID: charts[selectedChart].id}
      setTree(Object.assign({}, tree))
    }
  }

  const removeTreeElement = (event) => {
    let treeNode = getTreeNode()
    if (treeNode !== undefined) {
      if (treeNode.split) {
        delete treeNode["split"]
      }
      if (treeNode.generatorName) {
        delete treeNode["generatorName"]
      }
      if (treeNode.args) {
        delete treeNode["args"]
      }
      setTree(Object.assign({}, tree))
    }
  }

  function MinusSquare(props) {
    return (
      <SvgIcon fontSize="inherit" style={{width: 14, height: 14}} {...props}>
        {/* tslint:disable-next-line: max-line-length */}
        <path
          d="M22.047 22.074v0 0-20.147 0h-20.12v0 20.147 0h20.12zM22.047 24h-20.12q-.803 0-1.365-.562t-.562-1.365v-20.147q0-.776.562-1.351t1.365-.575h20.147q.776 0 1.351.575t.575 1.351v20.147q0 .803-.575 1.365t-1.378.562v0zM17.873 11.023h-11.826q-.375 0-.669.281t-.294.682v0q0 .401.294 .682t.669.281h11.826q.375 0 .669-.281t.294-.682v0q0-.401-.294-.682t-.669-.281z"/>
      </SvgIcon>
    );
  }

  function PlusSquare(props) {
    return (
      <SvgIcon fontSize="inherit" style={{width: 14, height: 14}} {...props}>
        {/* tslint:disable-next-line: max-line-length */}
        <path
          d="M22.047 22.074v0 0-20.147 0h-20.12v0 20.147 0h20.12zM22.047 24h-20.12q-.803 0-1.365-.562t-.562-1.365v-20.147q0-.776.562-1.351t1.365-.575h20.147q.776 0 1.351.575t.575 1.351v20.147q0 .803-.575 1.365t-1.378.562v0zM17.873 12.977h-4.923v4.896q0 .401-.281.682t-.682.281v0q-.375 0-.669-.281t-.294-.682v-4.896h-4.923q-.401 0-.682-.294t-.281-.669v0q0-.401.281-.682t.682-.281h4.923v-4.896q0-.401.294-.682t.669-.281v0q.401 0 .682.281t.281.682v4.896h4.923q.401 0 .682.281t.281.682v0q0 .375-.281.669t-.682.294z"/>
      </SvgIcon>
    );
  }

  function CloseSquare(props) {
    return (
      <SvgIcon
        className="close"
        fontSize="inherit"
        style={{width: 14, height: 14}}
        {...props}
      >
        {/* tslint:disable-next-line: max-line-length */}
        <path
          d="M17.485 17.512q-.281.281-.682.281t-.696-.268l-4.12-4.147-4.12 4.147q-.294.268-.696.268t-.682-.281-.281-.682.294-.669l4.12-4.147-4.12-4.147q-.294-.268-.294-.669t.281-.682.682-.281.696 .268l4.12 4.147 4.12-4.147q.294-.268.696-.268t.682.281 .281.669-.294.682l-4.12 4.147 4.12 4.147q.294.268 .294.669t-.281.682zM22.047 22.074v0 0-20.147 0h-20.12v0 20.147 0h20.12zM22.047 24h-20.12q-.803 0-1.365-.562t-.562-1.365v-20.147q0-.776.562-1.351t1.365-.575h20.147q.776 0 1.351.575t.575 1.351v20.147q0 .803-.575 1.365t-1.378.562v0z"/>
      </SvgIcon>
    );
  }

  let ephemeralExpanded = []

  const createPartialTree = (childNode, relativePath, displayName) => {
    ephemeralExpanded.push(relativePath.join(treePathSeparator))
    //TODO: Add IDs to expanded
    if (childNode.split) {
      //TODO: Only use last component of relativePath
      const clonedPath1 = Object.assign([], relativePath)
      clonedPath1.push("split")
      clonedPath1.push("c1")
      const clonedPath2 = Object.assign([], relativePath)
      clonedPath2.push("split")
      clonedPath2.push("c2")
      return (<TreeItem nodeId={relativePath.join(treePathSeparator)} label={displayName}>
        {createPartialTree(childNode.split.c1, clonedPath1, childNode.split.horizontal ? "Links" : "Oben")}
        {createPartialTree(childNode.split.c2, clonedPath2, childNode.split.horizontal ? "Rechts" : "Unten")}
      </TreeItem>)
    } else if (childNode.generatorName) {
      //TODO: Add display name of chart
      const chartName = charts["" + childNode.args.chartID] ? charts["" + childNode.args.chartID].chart_name : ("chart_" + childNode.args.chartID)
      return (<TreeItem nodeId={relativePath.join(treePathSeparator)} label={displayName + ": " + chartName}/>)
    } else {
      return (<TreeItem nodeId={relativePath.join(treePathSeparator)} label={displayName + ": leer"}/>)
    }
  }


  let createTreeview = undefined;
  if(tree) {
    createTreeview = createPartialTree(tree.c1, ["c1"], "Dashboard")
    updateExpanded(ephemeralExpanded)
  }

  useEffect(() => {
    //TODO: disable/enable buttons
  }, [selected])

  useEffect(() => {
    getCharts()
  }, [])

  return (
    <React.Fragment>
      <TreeView
        aria-label="file system navigator"
        defaultCollapseIcon={<MinusSquare/>}
        defaultExpandIcon={<PlusSquare/>}
        defaultEndIcon={<CloseSquare/>}
        expanded={expanded}
        selected={selected}
        onNodeSelect={(event, node) => {
          setSelected([node])
        }}
        sx={{flexGrow: 1, maxWidth: 400,}}
      >
        {createTreeview}
      </TreeView>
      <div>
        <IconButton onClick={verticalSplit}>
          <HorizontalSplit/>
        </IconButton>
        <IconButton onClick={horizontalSplit}>
          <VerticalSplit/>
        </IconButton>
      </div>
      <div>
        <IconButton onClick={addChart}>
          <AddBoxIcon/>
        </IconButton>
        <Select
          id="demo-controlled-time-open-select"
          value={selectedChart}
          onChange={(event, child) => {
            setSelectedChart(event.target.value)
          }}
        >
          {Object.entries(charts).map(([chart_id, chart]) => {
            return (<MenuItem value={chart_id}> {chart.chart_name} </MenuItem>)
          })}
        </Select>
      </div>
      <div>
        <IconButton onClick={removeTreeElement}>
          <RemoveIcon/>
        </IconButton>
      </div>
      <div>
        <IconButton onClick={onSave}>
          <SaveIcon />
        </IconButton>
      </div>
    </React.Fragment>
  )
};