import React from "react";
import {APIContext} from './callWrapper'
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import Typography from "@material-ui/core/Typography";
import {Button} from "@material-ui/core";
import deepEqual from 'deep-equal'
import {Link, useNavigate} from "react-router-dom";
import Modal from "@material-ui/core/Modal";
import {makeStyles} from "@material-ui/core/styles";
import DeleteIcon from '@material-ui/icons/Delete';


const drawerWidth = 240;
const useStyles = makeStyles((theme) => ({
  root: {
    display: 'flex',
    'min-height': '100vh',
    position: 'relative',
    'padding-bottom': '60px',
  },
  appBar: {
    zIndex: theme.zIndex.drawer + 1,
    transition: theme.transitions.create(['width', 'margin'], {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.leavingScreen,
    }),
  },
  appBarShift: {
    marginLeft: drawerWidth,
    width: `calc(100% - ${drawerWidth}px)`,
    transition: theme.transitions.create(['width', 'margin'], {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.enteringScreen,
    }),
  },
  menuButton: {
    marginRight: 36,
  },
  hide: {
    display: 'none',
  },
  drawer: {
    width: drawerWidth,
    flexShrink: 0,
    whiteSpace: 'nowrap',
  },
  drawerOpen: {
    width: drawerWidth,
    transition: theme.transitions.create('width', {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.enteringScreen,
    }),
  },
  drawerClose: {
    transition: theme.transitions.create('width', {
      easing: theme.transitions.easing.sharp,
      duration: theme.transitions.duration.leavingScreen,
    }),
    overflowX: 'hidden',
    width: theme.spacing(7) + 1,
    [theme.breakpoints.up('sm')]: {
      width: theme.spacing(9) + 1,
    },
  },
  toolbar: {
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'flex-end',
    padding: theme.spacing(0, 1),
    // necessary for content to be below app bar
    ...theme.mixins.toolbar,
  },
  content: {
    flexGrow: 1,
    padding: theme.spacing(3),
  },
  paper: {
    position: 'absolute',
    width: 400,
    backgroundColor: theme.palette.background.paper,
    border: '2px solid #000',
    boxShadow: theme.shadows[5],
    padding: theme.spacing(2, 4, 3),
  },
}));

const UserDashboard = () => {
	
    const client = React.useContext(APIContext)
    const classes = useStyles();
    const history = useNavigate()
    const [dashboards, setDashboards] = React.useState([])
    const [deleteModalConfig, setDeleteModalConfig] = React.useState({open:false, dashboard: null})

    const closeModal = () => {
      setDeleteModalConfig({open:false, dashboard: null})

    }

    const openModal = (dashboard) => {
      setDeleteModalConfig({open:true, dashboard: dashboard})
    }

    const getUserDashboards = () => {
      console.log("in getUserDashboards");
      client.getDashboards().then((new_dashboards) => {
        if (!deepEqual(dashboards, new_dashboards)) {
          console.log("in updateDashboards")
          console.log(new_dashboards)
          setDashboards(new_dashboards);
        }
      }).catch((error) => {
        console.log(error.response);
      });
    }

    const updateDashboards = () => {
      client.getDashboards().then((new_dashboards) => {
        if (!deepEqual(dashboards, new_dashboards)) {
          console.log("in updateDashboards")
          console.log(new_dashboards)
          setDashboards(new_dashboards);
        }
      }).catch((error) => {
        console.log(error.response);
      });
    }

    const getVisibilityName = (visibilityId) => {
      switch(visibilityId) {
        case 0:
          return "Privat";
        case 1:
          return "Geteilt";
        case 2: 
          return "Teil-Öffentlich";
        case 3:
          return "Öffentlich";
      }
    }

    React.useEffect(() => {
      console.log("State changed");
      updateDashboards();
    })

    React.useEffect(() => {
      //Run only once on mount
      getUserDashboards();
    }, []);

    React.useEffect(() => {
      //Run only once on mount
    }, []);

  return (
      <React.Fragment>
        <Modal
        open={deleteModalConfig.open}
        onClose={closeModal}
        aria-labelledby="simple-modal-title"
        aria-describedby="simple-modal-description"
      >
        <div style={{top: '30%', left: '30%',}} className={classes.paper}>
          <Typography component="h2" variant="h6" color="primary" gutterBottom> {"Dashboard "+ ((deleteModalConfig.dashboard != null) ? deleteModalConfig.dashboard.name : "") + " wirklich löschen?"} </Typography>
          <Button variant="contained" component="label" color="primary" onClick={() => {
            client.deleteDashboard(deleteModalConfig.dashboard.id).then(updateDashboards).catch( (error) => {
              //TODO: Inform user about error
              console.error(error)
            }).finally(closeModal)
          }
          }> Löschen</Button>
          <Button variant="contained" component="label" color="primary" onClick={() => {
            closeModal();
          }
          }> Abbrechen </Button>
        </div>
      </Modal>
        <Typography component="h2" variant="h6" gutterBottom> Dashboards </Typography>
        <Table>
          <TableHead>
            <TableRow>
              <TableCell>Dashboardname</TableCell>
              <TableCell>Öffentlich</TableCell>
              <TableCell>Löschen</TableCell>
            </TableRow>
          </TableHead>
          <TableBody>
            {dashboards.map((dashboard) => (
              <TableRow>
                <TableCell component={Link} to={{pathname: `/dashboard/${dashboard.id}`,state: {} }}> {dashboard.name}</TableCell>
                <TableCell>
                  {getVisibilityName(dashboard.visibility)}
                </TableCell>
                <TableCell> {(client.user?.id === dashboard.owner)
                  ? <DeleteIcon variant="contained" color="secondary" onClick={() => {
                    openModal(dashboard)
                  }
                  } />
                  : ""}
                </TableCell>
              </TableRow>
            ))}
          </TableBody>
        </Table>
      </React.Fragment>
    );

};


export default UserDashboard;
