import React from 'react';
import {APIContext} from './callWrapper';
import Button from '@material-ui/core/Button';
import Dialog from '@material-ui/core/Dialog';
import DialogActions from '@material-ui/core/DialogActions';
import DialogContent from '@material-ui/core/DialogContent';
import DialogContentText from '@material-ui/core/DialogContentText';
import DialogTitle from '@material-ui/core/DialogTitle';
import FormControl from '@material-ui/core/FormControl';
import InputLabel from '@material-ui/core/InputLabel';
import Select from '@material-ui/core/Select';
import List from '@material-ui/core/List';
import ListItemAvatar from '@material-ui/core/ListItemAvatar';
import ListItemText from '@material-ui/core/ListItemText';
import MuiListItem from "@material-ui/core/ListItem";
import MenuItem from '@material-ui/core/MenuItem';
import FolderIcon from '@material-ui/icons/Folder';
import Avatar from '@material-ui/core/Avatar';
import Grid from '@material-ui/core/Grid';
import { makeStyles, withStyles  } from '@material-ui/core/styles';
import deepEqual from 'deep-equal'

const useStyles = makeStyles((theme) => ({
  h1: {
    '& .MuiTypography-root': {
        color: "#0F3F7B"},
  },
  dialogWindow: {
  },
  formControl: {
    margin: theme.spacing(1),
    minWidth: 120,
  },
}));

const ListItem = withStyles({
  root: {
    "&$selected": {
      backgroundColor: "#0F3F7B",
      color: "white",
      "& .MuiListItemIcon-root": {
        color: "white"
      }
    },
    "&$selected:hover": {
      backgroundColor: "#D6DCE1",
      color: "#192E4A",
      "& .MuiListItemIcon-root": {
        color: "#192E4A"
      }
    },
    "&$selected:active":  {
      backgroundColor: "#192E4A",
      color: "white",
      "& .MuiListItemIcon-root": {
        color: "white"
      }
    },
    "&$selected:focus":  {
      backgroundColor: "#192E4A",
      color: "white",
      "& .MuiListItemIcon-root": {
        color: "white"
      }
    },
    "&:hover": {
      backgroundColor: "#D6DCE1",
      color: "#192E4A",
      "& .MuiListItemIcon-root": {
        color: "#192E4A"
      }
    },
    "&:active": {
      backgroundColor: "#192E4A",
      color: "white",
      "& .MuiListItemIcon-root": {
        color: "white"
      }
    },
    "&:focus": {
      backgroundColor: "#192E4A",
      color: "white",
      "& .MuiListItemIcon-root": {
        color: "white"
      }
    }
  },
  selected: {}
})(MuiListItem);

/* It's a scrolldialog for adding charts in a dashboard. */
const AddChartsDialog = (props) => {
  const classes = useStyles();
  const client = React.useContext(APIContext);
  const [open, setOpen] = React.useState(false);
  const [scroll, setScroll] = React.useState('paper');
  const [selectedIndex, setSelectedIndex] = React.useState(1);
  const [addedCharts, setAddedCharts] = React.useState([]);
  const [committedState, setstate] = React.useState({
      dropdown: {
        'group-creation-visibility-select': {
          open: false,
          value: 0,
        },
        'demo-controlled-time-open-select': {
                open: false,
                value: "",
        },
        'demo-controlled-charttype-open-select': {
            open: false,
            value: 10,
        }
        ,
        'demo-controlled-filter-open-select': {
            open: false,
            value: 10,
        }
      },
      new_dashboard_name: "",
      dashboard_charts: [],
      last_opened: "",
      previous: {}
  });
  const uncommittedState = JSON.parse(JSON.stringify(committedState));
  const descriptionElementRef = React.useRef(null);
  let onceBuild = 0;

  /**
   * Changes uncommitted state to a comitted one.
   */
  const commitState = () => { setstate(uncommittedState); };
  
  /**
   * Checks if a value in the select-element was changed.
   * @return if a value was changed.
   */
  const valuesChanged = () => {
    for (let [name, list] of Object.entries(uncommittedState.dropdown)) {
        if ( !(
            uncommittedState.previous &&
            uncommittedState.previous.dropdown &&
            uncommittedState.previous.dropdown[name] &&
            uncommittedState.previous.dropdown[name].value === list.value)) {
            return true;
        }
    }
    return false;
  }

  /**
   * Handles the change of the select-element an its' value.
   * @param {event} event
   */
  const handleChange = (event) => {
    let list = uncommittedState.dropdown[uncommittedState.last_opened];
    list.value = event.target.value;
    commitState()
  };

  /**
   * Handles the close of the dialog.
   * @param {event} event
   */
  const handleCloseDialog = (event) => {
    setOpen(false);
  };

  /**
   * Handles the open of the select-element last_opened.
   */
  const handleOpen = (event) => {
    uncommittedState.last_opened = event.target.id;
    uncommittedState.dropdown[uncommittedState.last_opened]['open'] = true;
    commitState()
  };

  /**
   * Handles the close of the select-element last_opened.
   */
  const handleClose = (event) => {
      let list = uncommittedState.dropdown[uncommittedState.last_opened]
      list.open = false;
      commitState()
  };

  /**
   * Handles the open-status of "Charts hinzufügen". 
   */
  const handleClickOpen = (event) => {
    setOpen(true);
  };

  const handleBestaetigen = () => {
    props.parentChartsCallback(addedCharts);
    setAddedCharts([]);
    setOpen(false);
  }

  /**
   * Gets all charts from server.
   * @return charts
   */
   const getCharts = () => {
    client.getCharts().then((charts) => {
      uncommittedState.dashboard_charts = charts;
      commitState();
    })
    .catch((error) => {
      console.log(error.response);
    })
   }

  /**
   * Creates a list with all available charts which can be added to the dashboard.
   * @return {HTMLElement} returns the chartslist 
   */
  const buildChartsList = () => {
    return (<List>
            {uncommittedState.dashboard_charts.map((chart) => (
              <ListItem 
                button 
                onClick={(event) => {handleListItemClick(event, uncommittedState.dashboard_charts.indexOf(chart)); clickChart(chart)}}>
                <ListItemAvatar>
                  <Avatar>
                    <FolderIcon/> {/* TODO: Insert User Avatar here*/}
                  </Avatar>
                </ListItemAvatar>
                <ListItemText primaryTypographyProps={{color: "primary"}} primary={chart.chart_name}/>
              </ListItem>
            ))}
          </List>) 
  };

  /**
   * Adds clicked chart in the list for building chartlist in CreateDashboard.
   * @param {object} user 
   */
  const clickChart = (chart) => {
    if (addedCharts.indexOf(chart) == -1) { 
      setAddedCharts(addedCharts.concat(chart)) 
    }   
  }

  /**
   * Handles the clickevent when clicking on a listitem (chart).
   * @param {event} event
   * @para, {number} index
   */
  const handleListItemClick = (event, index) => { setSelectedIndex(index); };

  /**
   * Updates the changes of the chartslist by the chosen filter options.
   * The result is seen as a filtered chartlist in the dialog.
   */
  const updatePreviews = () => {
    // Update the reference value, so valuesChanged returns false next time
    const cloned_state = JSON.parse(JSON.stringify(uncommittedState));
    delete cloned_state.previous;
    uncommittedState.previous = cloned_state;

    const param = {}
    if (uncommittedState.dropdown['demo-controlled-time-open-select']['value'] === 10) {
        let date = new Date();
        date.setDate(date.getDate() - 7);
        param['creation_time__gte'] = date.toISOString()
    }
    if (uncommittedState.dropdown['demo-controlled-time-open-select']['value'] === 20) {
        let date = new Date();
        date.setDate(date.getDate() - 30);
        param['creation_time__gte'] = date.toISOString()
    }
    switch (uncommittedState.dropdown['demo-controlled-charttype-open-select']['value']) {
        case 20:
            param['chart_type'] = 'piechart';
            break;
        case 30:
            param['chart_type'] = 'linechart';
            break;
        case 40:
            param['chart_type'] = 'chordchart';
            break;
        case 50:
            param['chart_type'] = 'bubblechart';
            break;

        case 60:
            param['chart_type'] = 'scatterchart';
            break;
        case 70:
            param['chart_type'] = 'hiveplot';
            break;
    }

    if(client) {
      client.getCharts(param['chart_type'], param['creation_time__gte'],undefined,undefined,undefined,true).then(charts => {
        if (!(deepEqual(charts, uncommittedState.dashboard_charts))) {
          //Only run if local data differed from most recent server response
          uncommittedState.dashboard_charts = charts;
          commitState();
        }
      });
    }
  }

  React.useEffect(() => {
    if (open) {
      const { current: descriptionElement } = descriptionElementRef;
      if (descriptionElement !== null) {
        descriptionElement.focus();
      }
    }
    onceBuild++;
  }, [open]);

  React.useEffect( () => {
    updatePreviews();
  })

  React.useEffect(() => {
    getCharts();
  }, [])

  return (
    <div>
      <Button onClick={() => handleClickOpen()} variant="contained" component="label" color="primary">Charts hinzufügen</Button>
      <Dialog
        open={open}
        onClose={handleCloseDialog}
        scroll={scroll}
        aria-labelledby="scroll-dialog-title"
        aria-describedby="scroll-dialog-description"
        className={classes.dialogWindow}
      >
        <DialogTitle id="scroll-dialog-title" className={classes.h1}>Fügen Sie Charts zu Ihrem Dashboard hinzu</DialogTitle>
        <DialogContent dividers={scroll === 'paper'}>
          <DialogContentText
            id="scroll-dialog-description"
            ref={descriptionElementRef}
            tabIndex={-1}
          >
            <Grid container spacing={3}>
              <Grid item xs>
                <FormControl className={classes.formControl} id="selectPopular">
              <InputLabel id="demo-controlled-time-open-select-label">Charts der letzten 30 Tagen</InputLabel>
              <Select
                labelId="demo-controlled-time-open-select-label"
                id="demo-controlled-time-open-select"
                open={uncommittedState.dropdown['demo-controlled-time-open-select']['open']}
                onClose={handleClose}
                onOpen={handleOpen}
                value={uncommittedState.dropdown['demo-controlled-time-open-select']['value']}
                onChange={handleChange}
              >
                <MenuItem value="">
                  <em>None</em>
                </MenuItem>
                <MenuItem value={10}>Charts der letzten 7 Tagen</MenuItem>
                <MenuItem value={20}>Charts der letzten 30 Tagen</MenuItem>
                <MenuItem value={30}>populärste Charts</MenuItem>
                <MenuItem value={40}>neusten Charts</MenuItem>
                <MenuItem value={50}>meist erstellten Charts</MenuItem>
              </Select>
          </FormControl>
              </Grid>
              <Grid item xs>
                <FormControl className={classes.formControl} id="selectCharts">
              <InputLabel id="demo-controlled-charttype-open-select-label">alle Charts</InputLabel>
              <Select
                labelId="demo-controlled-charttype-open-select-label"
                id="demo-controlled-charttype-open-select"
                open={uncommittedState.dropdown['demo-controlled-charttype-open-select']['open']}
                onClose={handleClose}
                onOpen={handleOpen}
                value={uncommittedState.dropdown['demo-controlled-charttype-open-select']['value']}
                onChange={handleChange}
              >
                <MenuItem value={10}>alle Charts</MenuItem>
                <MenuItem value={20}>Piecharts</MenuItem>
                <MenuItem value={30}>Linecharts</MenuItem>
                <MenuItem value={40}>Chordcharts</MenuItem>
                <MenuItem value={50}>Bubblecharts</MenuItem>
                <MenuItem value={60}>Scattercharts</MenuItem>
                <MenuItem value={70}>Hiveplots</MenuItem>
              </Select>
          </FormControl>
              </Grid>
              <Grid item xs>
                <FormControl className={classes.formControl} id="selectFilter">
              <InputLabel id="demo-controlled-filter-open-select-label">Filter nach</InputLabel>
              <Select
                labelId="demo-controlled-filter-open-select-label"
                id="demo-controlled-filter-open-select"
                open={uncommittedState.dropdown['demo-controlled-filter-open-select']['open']}
                onClose={handleClose}
                onOpen={handleOpen}
                value={uncommittedState.dropdown['demo-controlled-filter-open-select']['value']}
                onChange={handleChange}
              >
                <MenuItem value="">
                  <em>None</em>
                </MenuItem>
                <MenuItem value={10}>Nutzer</MenuItem>
                <MenuItem value={20}>Gruppen</MenuItem>
                <MenuItem value={30}>Collections</MenuItem>
                <MenuItem value={40}>Dashboards</MenuItem>
              </Select>
          </FormControl>
              </Grid>
            </Grid>
            {buildChartsList()}
          </DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={handleCloseDialog} color="primary">
            Abbrechen
          </Button>
          <Button onClick={handleBestaetigen} color="primary">
            Bestätigen
          </Button>
        </DialogActions>
      </Dialog>
    </div>
  );
};

export default AddChartsDialog; 
